@@ -5,7 +5,7 @@
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2007-2009 Tilman Benkert <thzs@gmx.net>
 	SPDX-FileCopyrightText: 2007-2010 Knut Franke <knut.franke@gmx.de>
-	SPDX-FileCopyrightText: 2011-2022 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2011-2023 Alexander Semke <alexander.semke@web.de>
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -29,7 +29,7 @@ class QXmlStreamWriter;
 
 /// Information about class inheritance
 /// enum values are chosen such that @verbatim inherits(base)@endverbatim
-/// returns true iff the class inherits from @verbatim base@endverbatim.
+/// returns true if the class inherits from @verbatim base@endverbatim.
 ///
 /// AspectType is used in GuiObserver to select the correct dock widget.
 enum class AspectType : quint64 {
@@ -44,21 +44,35 @@ enum class AspectType : quint64 {
 	Axis = 0x0210001,
 	CartesianPlotLegend = 0x0210002,
 	CustomPoint = 0x0210004,
-	Histogram = 0x0210008,
 	PlotArea = 0x0210010,
 	TextLabel = 0x0210020,
 	Image = 0x0210030,
 	ReferenceLine = 0x0210040,
 	ReferenceRange = 0x0210060,
 	InfoElement = 0x0210080,
-	BoxPlot = 0x0210100,
+
+	// bar plots
 	BarPlot = 0x0210200,
+	LollipopPlot = 0x0210400,
+
+	// statistical plots
+	Histogram = 0x0210008,
+	BoxPlot = 0x0210100,
+	QQPlot = 0x0210800,
+	KDEPlot = 0x0210802,
+
+	// continious improvement plots
+	ProcessBehaviorChart = 0x0211000,
+	RunChart = 0x0211001,
+
 	WorksheetElementContainer = 0x0220000,
 	AbstractPlot = 0x0221000,
 	CartesianPlot = 0x0221001,
 	WorksheetElementGroup = 0x0222000,
 	XYCurve = 0x0240000,
 	XYEquationCurve = 0x0240001,
+
+	// analysis curves
 	XYAnalysisCurve = 0x0280000,
 	XYConvolutionCurve = 0x0280001,
 	XYCorrelationCurve = 0x0280002,
@@ -71,6 +85,7 @@ enum class AspectType : quint64 {
 	XYIntegrationCurve = 0x0280100,
 	XYSmoothCurve = 0x0280200,
 	XYHilbertTransformCurve = 0x0280400,
+	XYFunctionCurve = 0x0280800,
 
 	AbstractPart = 0x0400000,
 	AbstractDataSource = 0x0410000,
@@ -78,7 +93,8 @@ enum class AspectType : quint64 {
 	Spreadsheet = 0x0412000,
 	LiveDataSource = 0x0412001,
 	MQTTTopic = 0x0412002,
-	CantorWorksheet = 0x0420001,
+	StatisticsSpreadsheet = 0x0412004,
+	Notebook = 0x0420001,
 	Datapicker = 0x0420002,
 	DatapickerImage = 0x0420004,
 	Note = 0x0420008,
@@ -167,6 +183,8 @@ public:
 			return QStringLiteral("XYCurve");
 		case AspectType::XYEquationCurve:
 			return QStringLiteral("XYEquationCurve");
+		case AspectType::XYFunctionCurve:
+			return QStringLiteral("XYFunctionCurve");
 		case AspectType::XYAnalysisCurve:
 			return QStringLiteral("XYAnalysisCurve");
 		case AspectType::XYConvolutionCurve:
@@ -195,6 +213,16 @@ public:
 			return QStringLiteral("BarPlot");
 		case AspectType::BoxPlot:
 			return QStringLiteral("BoxPlot");
+		case AspectType::QQPlot:
+			return QStringLiteral("QQPlot");
+		case AspectType::KDEPlot:
+			return QStringLiteral("KDEPlot");
+		case AspectType::LollipopPlot:
+			return QStringLiteral("LollipopPlot");
+		case AspectType::ProcessBehaviorChart:
+			return QStringLiteral("ProcessBehaviorChart");
+		case AspectType::RunChart:
+			return QStringLiteral("RunChart");
 		case AspectType::AbstractPart:
 			return QStringLiteral("AbstractPart");
 		case AspectType::AbstractDataSource:
@@ -203,12 +231,14 @@ public:
 			return QStringLiteral("Matrix");
 		case AspectType::Spreadsheet:
 			return QStringLiteral("Spreadsheet");
+		case AspectType::StatisticsSpreadsheet:
+			return QStringLiteral("StatisticsSpreadsheet");
 		case AspectType::LiveDataSource:
 			return QStringLiteral("LiveDataSource");
 		case AspectType::MQTTTopic:
 			return QStringLiteral("MQTTTopic");
-		case AspectType::CantorWorksheet:
-			return QStringLiteral("CantorWorksheet");
+		case AspectType::Notebook:
+			return QStringLiteral("Notebook");
 		case AspectType::Datapicker:
 			return QStringLiteral("Datapicker");
 		case AspectType::DatapickerImage:
@@ -241,13 +271,15 @@ public:
 	}
 
 	QString name() const;
+	QUuid uuid() const;
+	void setSuppressWriteUuid(bool);
 	QString comment() const;
 	void setCreationTime(const QDateTime&);
 	QDateTime creationTime() const;
 	virtual Project* project();
 	virtual QString path() const;
 	void setHidden(bool);
-	bool hidden() const;
+	bool isHidden() const;
 	void setFixed(bool);
 	bool isFixed() const;
 	void setSelected(bool);
@@ -257,25 +289,34 @@ public:
 	bool isLoading() const;
 	virtual QIcon icon() const;
 	virtual QMenu* createContextMenu();
+	void setProjectChanged(bool);
 
 	AspectType type() const;
 	bool inherits(AspectType type) const;
 
 	// functions related to the handling of the tree-like project structure
 	AbstractAspect* parentAspect() const;
-	AbstractAspect* parent(AspectType type) const;
+	AbstractAspect* parent(AspectType) const;
 	void setParentAspect(AbstractAspect*);
 	Folder* folder();
 	bool isDescendantOf(AbstractAspect* other);
 	void addChild(AbstractAspect*);
 	void addChildFast(AbstractAspect*);
 	virtual void finalizeAdd(){};
-	QVector<AbstractAspect*> children(AspectType type, ChildIndexFlags flags = {}) const;
+	QVector<AbstractAspect*> children(AspectType, ChildIndexFlags = {}) const;
+	void insertChild(AbstractAspect* child, int index);
 	void insertChildBefore(AbstractAspect* child, AbstractAspect* before);
 	void insertChildBeforeFast(AbstractAspect* child, AbstractAspect* before);
 	void reparent(AbstractAspect* newParent, int newIndex = -1);
+	/*!
+	 * \brief removeChild
+	 * Removing child aspect using an undo command
+	 * \param parent If parent is not nullptr the command will not be executed, but the parent must be executed
+	 * to indirectly execute the created undocommand
+	 */
 	void removeChild(AbstractAspect*);
 	void removeAllChildren();
+	void moveChild(AbstractAspect*, int steps);
 	virtual QVector<AbstractAspect*> dependsOn() const;
 
 	virtual QVector<AspectType> pasteTypes() const;
@@ -299,7 +340,7 @@ public:
 	QVector<T*> children(ChildIndexFlags flags = {}) const {
 		QVector<T*> result;
 		for (auto* child : children()) {
-			if (flags & ChildIndexFlag::IncludeHidden || !child->hidden()) {
+			if (flags & ChildIndexFlag::IncludeHidden || !child->isHidden()) {
 				T* i = dynamic_cast<T*>(child);
 				if (i)
 					result << i;
@@ -316,7 +357,7 @@ public:
 		int i = 0;
 		for (auto* child : children()) {
 			T* c = dynamic_cast<T*>(child);
-			if (c && (flags & ChildIndexFlag::IncludeHidden || !child->hidden()) && index == i++)
+			if (c && (flags & ChildIndexFlag::IncludeHidden || !child->isHidden()) && index == i++)
 				return c;
 		}
 		return nullptr;
@@ -337,7 +378,7 @@ public:
 		int result = 0;
 		for (auto* child : children()) {
 			T* i = dynamic_cast<T*>(child);
-			if (i && (flags & ChildIndexFlag::IncludeHidden || !child->hidden()))
+			if (i && (flags & ChildIndexFlag::IncludeHidden || !child->isHidden()))
 				result++;
 		}
 		return result;
@@ -350,7 +391,7 @@ public:
 			if (child == c)
 				return index;
 			T* i = dynamic_cast<T*>(c);
-			if (i && (flags & ChildIndexFlag::IncludeHidden || !c->hidden()))
+			if (i && (flags & ChildIndexFlag::IncludeHidden || !c->isHidden()))
 				index++;
 		}
 		return -1;
@@ -374,14 +415,16 @@ public:
 	virtual void save(QXmlStreamWriter*) const = 0;
 	virtual bool load(XmlStreamReader*, bool preview) = 0;
 	void setPasted(bool);
-	bool pasted() const;
+	bool isPasted() const;
 
 	static AspectType clipboardAspectType(QString&);
 	static QString uniqueNameFor(const QString& name, const QStringList& names);
 
+	typedef AbstractAspectPrivate Private;
+
 protected:
 	void info(const QString& text) {
-		emit statusInfo(text);
+		Q_EMIT statusInfo(text);
 	}
 
 	// serialization/deserialization
@@ -400,12 +443,12 @@ private:
 	void connectChild(AbstractAspect*);
 
 public Q_SLOTS:
-	bool setName(const QString&, NameHandling handling = NameHandling::AutoUnique);
+	bool setName(const QString&, NameHandling handling = NameHandling::AutoUnique, QUndoCommand* parent = nullptr);
 	void setComment(const QString&);
 	void remove();
-	void copy() const;
+	void copy();
 	void duplicate();
-	void paste(bool duplicate = false);
+	void paste(bool duplicate = false, int index = -1);
 
 private Q_SLOTS:
 	void moveUp();
@@ -418,10 +461,35 @@ protected Q_SLOTS:
 Q_SIGNALS:
 	void aspectDescriptionAboutToChange(const AbstractAspect*);
 	void aspectDescriptionChanged(const AbstractAspect*);
-	void aspectAboutToBeAdded(const AbstractAspect* parent, const AbstractAspect* before, const AbstractAspect* child);
-	void aspectAdded(const AbstractAspect*);
+	/*!
+	 * \brief aspectAboutToBeAdded
+	 * Signal indicating a new child was added at position \p index. Do not connect to both variants of aspectAboutToBeAdded!
+	 * \param parent
+	 * \param index Position of the new aspect
+	 * \param child
+	 */
+	void childAspectAboutToBeAdded(const AbstractAspect* parent, int index, const AbstractAspect* child);
+	/*!
+	 * \brief aspectAboutToBeAdded
+	 * \param parent
+	 * \param before aspect one position before the child
+	 * \param child
+	 */
+	void childAspectAboutToBeAdded(const AbstractAspect* parent, const AbstractAspect* before, const AbstractAspect* child);
+	void childAspectAdded(const AbstractAspect*);
+	/*!
+	 * \brief aspectAboutToBeRemoved
+	 * Called from the parent if a child is being removed
+	 */
+	void childAspectAboutToBeRemoved(const AbstractAspect* child);
+	void childAspectRemoved(const AbstractAspect* parent, const AbstractAspect* before, const AbstractAspect* child);
+	/*!
+	 * \brief aspectAboutToBeRemoved
+	 * Called by the aspect itself when it's being removed
+	 */
 	void aspectAboutToBeRemoved(const AbstractAspect*);
-	void aspectRemoved(const AbstractAspect* parent, const AbstractAspect* before, const AbstractAspect* child);
+	void childAspectAboutToBeMoved(const AbstractAspect*, int destinationRow);
+	void childAspectMoved();
 	void aspectHiddenAboutToChange(const AbstractAspect*);
 	void aspectHiddenChanged(const AbstractAspect*);
 	void statusInfo(const QString&);
@@ -450,9 +518,12 @@ public:
 	int mRetransformCalled{0};
 
 #define trackRetransformCalled(suppressed)                                                                                                                     \
-	emit q->retransformCalledSignal(q, suppressed);                                                                                                            \
+	Q_EMIT q->retransformCalledSignal(q, suppressed);                                                                                                          \
 	if (!suppressed)                                                                                                                                           \
 		q->mRetransformCalled += 1;
+
+	friend class AbstractAspectTest;
+	friend class InfoElementTest;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(AbstractAspect::ChildIndexFlags)
