@@ -12,8 +12,9 @@
 #include "backend/lib/XmlStreamReader.h"
 #include "backend/matrix/Matrix.h"
 #include "backend/spreadsheet/Spreadsheet.h"
-#include "commonfrontend/workbook/WorkbookView.h"
-#include "kdefrontend/spreadsheet/ExportSpreadsheetDialog.h"
+#ifndef SDK
+#include "frontend/workbook/WorkbookView.h"
+#endif
 
 #include <KLocalizedString>
 #include <QIcon>
@@ -42,6 +43,7 @@ QMenu* Workbook::createContextMenu() {
 }
 
 QWidget* Workbook::view() const {
+#ifndef SDK
 	if (!m_partView) {
 		m_view = new WorkbookView(const_cast<Workbook*>(this));
 		m_partView = m_view;
@@ -50,6 +52,9 @@ QWidget* Workbook::view() const {
 		});
 	}
 	return m_partView;
+#else
+	return nullptr;
+#endif
 }
 
 bool Workbook::exportView() const {
@@ -92,6 +97,7 @@ bool Workbook::printPreview() const {
 }
 
 Spreadsheet* Workbook::currentSpreadsheet() const {
+#ifndef SDK
 	if (!m_view)
 		return nullptr;
 
@@ -100,10 +106,12 @@ Spreadsheet* Workbook::currentSpreadsheet() const {
 		auto* aspect = child<AbstractAspect>(index);
 		return dynamic_cast<Spreadsheet*>(aspect);
 	}
+#endif
 	return nullptr;
 }
 
 Matrix* Workbook::currentMatrix() const {
+#ifndef SDK
 	if (!m_view)
 		return nullptr;
 
@@ -112,6 +120,7 @@ Matrix* Workbook::currentMatrix() const {
 		auto* aspect = child<AbstractAspect>(index);
 		return dynamic_cast<Matrix*>(aspect);
 	}
+#endif
 	return nullptr;
 }
 
