@@ -13,13 +13,14 @@
 #define COLUMN_H
 
 #include "backend/core/AbstractColumn.h"
+#include "backend/nsl/nsl_sf_stats.h"
+#include <QPixmap>
 
 class AbstractSimpleFilter;
 class CartesianPlot;
 class ColumnStringIO;
 class QAction;
 class QActionGroup;
-class XmlStreamReader;
 class ColumnPrivate;
 class ColumnSetGlobalFormulaCmd;
 
@@ -66,6 +67,7 @@ public:
 	bool isReadOnly() const override;
 	void resizeTo(int);
 	int rowCount() const override;
+	int rowCount(double min, double max) const override;
 	int availableRowCount(int max = -1) const override;
 	int width() const;
 	void setWidth(const int);
@@ -73,12 +75,11 @@ public:
 	AbstractSimpleFilter* outputFilter() const;
 	ColumnStringIO* asStringColumn() const;
 
-	void setFormula(const QString& formula, const QStringList& variableNames, const QVector<Column*>& columns, bool autoUpdate);
+	// functions related to the formula used to generate column values
+	void setFormula(const QString& formula, const QStringList& variableNames, const QVector<Column*>& columns, bool autoUpdate = false, bool autoResize = true);
 	QString formula() const;
+	void clearFormula();
 	struct FormulaData {
-#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0)) // required to use in QVector
-		FormulaData() = default;
-#endif
 		FormulaData(const QString& variableName, const QString& columnPath)
 			: m_variableName(variableName)
 			, m_columnPath(columnPath) {
@@ -124,7 +125,22 @@ public:
 	void setFormulVariableColumnsPath(int index, const QString& path);
 	void setFormulVariableColumn(int index, Column*);
 	bool formulaAutoUpdate() const;
+	bool formulaAutoResize() const;
 
+	// functions related to the settings used to generate random values
+	struct RandomValuesData {
+		RandomValuesData() = default;
+		bool available{false};
+		nsl_sf_stats_distribution distribution{nsl_sf_stats_gaussian};
+		double parameter1{1.};
+		double parameter2{1.};
+		double parameter3{1.};
+		ulong seed{0}; // use 0 for "no fixed seed"
+	};
+	void setRandomValuesData(const RandomValuesData&);
+	RandomValuesData randomValuesData() const;
+
+	// functions related to the cells/rows formulas
 	QString formula(int) const override;
 	QVector<Interval<int>> formulaIntervals() const override;
 	void setFormula(const Interval<int>&, const QString&) override;
@@ -135,12 +151,16 @@ public:
 	void* data() const;
 	void setData(void*);
 	bool hasValues() const;
-	bool hasValueLabels() const;
+	bool hasValueAt(int row) const;
+	bool valueLabelsInitialized() const;
+	double valueLabelsMinimum() const;
+	double valueLabelsMaximum() const;
+	void setLabelsMode(ColumnMode mode);
 	void removeValueLabel(const QString&);
-	void clearValueLabels();
+	void valueLabelsRemoveAll();
 
 	Properties properties() const override;
-	void invalidateProperties();
+	void invalidateProperties() override;
 
 	void setFromColumn(int, AbstractColumn*, int);
 	QString textAt(int) const override;
@@ -149,8 +169,6 @@ public:
 	void replaceTexts(int, const QVector<QString>&) override;
 	int dictionaryIndex(int row) const override;
 	const QMap<QString, int>& frequencies() const;
-	void addValueLabel(const QString&, const QString&);
-	const QMap<QString, QString>& textValueLabels();
 
 	QDate dateAt(int) const override;
 	void setDateAt(int, QDate) override;
@@ -160,54 +178,72 @@ public:
 	QDateTime dateTimeAt(int) const override;
 	void setDateTimeAt(int, const QDateTime&) override;
 	void replaceDateTimes(int, const QVector<QDateTime>&) override;
-	void addValueLabel(const QDateTime&, const QString&);
-	const QMap<QDateTime, QString>& dateTimeValueLabels();
 
 	double doubleAt(int) const override;
 	double valueAt(int) const override;
 	void setValues(const QVector<double>&);
 	void setValueAt(int, double) override;
 	void replaceValues(int, const QVector<double>&) override;
-	void addValueLabel(double, const QString&);
-	const QMap<double, QString>& valueLabels();
 
 	int integerAt(int) const override;
 	void setIntegers(const QVector<int>&);
 	void setIntegerAt(int, int) override;
 	void replaceInteger(int, const QVector<int>&) override;
-	void addValueLabel(int, const QString&);
-	const QMap<int, QString>& intValueLabels();
 
 	qint64 bigIntAt(int) const override;
 	void setBigIntAt(int, qint64) override;
 	void setBigInts(const QVector<qint64>&);
 	void replaceBigInt(int, const QVector<qint64>&) override;
-	void addValueLabel(qint64, const QString&);
-	const QMap<qint64, QString>& bigIntValueLabels();
 
 	double maximum(int count = 0) const override;
 	double maximum(int startIndex, int endIndex) const override;
 	double minimum(int count = 0) const override;
 	double minimum(int startIndex, int endIndex) const override;
-	static int calculateMaxSteps(unsigned int value);
-	static int indexForValue(double x, QVector<double>& column, Properties properties = Properties::No);
-	static int indexForValue(const double x, const QVector<QPointF>& column, Properties properties = Properties::No);
-	static int indexForValue(double x, QVector<QLineF>& lines, Properties properties = Properties::No);
-	int indexForValue(double x) const override;
+	static int indexForValue(double x, QVector<double>& column, Properties properties, bool smaller);
+	static int indexForValue(const double x, const QVector<QPointF>& column, Properties properties, bool smaller);
+	static int indexForValue(double x, QVector<QLineF>& lines, Properties properties, bool smaller);
+	int indexForValue(double x, bool smaller) const override;
 	bool indicesMinMax(double v1, double v2, int& start, int& end) const override;
 
-	void setChanged();
-	void setSuppressDataChangedSignal(const bool);
-
 	void addUsedInPlots(QVector<CartesianPlot*>&);
 
+	// Value Labels
+	template<typename T>
+	struct ValueLabel {
+		T value;
+		QString label;
+	};
+	AbstractColumn::ColumnMode labelsMode() const;
+	int valueLabelsCount() const;
+	int valueLabelsCount(double min, double max) const;
+	int valueLabelsIndexForValue(double x, bool smaller) const;
+	double valueLabelsValueAt(int index) const;
+	QString valueLabelAt(int index) const;
+	void addValueLabel(qint64, const QString&);
+	const QVector<ValueLabel<qint64>>* bigIntValueLabels() const;
+	void addValueLabel(int, const QString&);
+	const QVector<ValueLabel<int>>* intValueLabels() const;
+	void addValueLabel(double, const QString&);
+	const QVector<ValueLabel<double>>* valueLabels() const;
+	void addValueLabel(const QDateTime&, const QString&);
+	const QVector<ValueLabel<QDateTime>>* dateTimeValueLabels() const;
+	void addValueLabel(const QString&, const QString&);
+	const QVector<ValueLabel<QString>>* textValueLabels() const;
+
 	void save(QXmlStreamWriter*) const override;
 	bool load(XmlStreamReader*, bool preview) override;
 	void finalizeLoad();
 
+	typedef ColumnPrivate Private;
+
 public Q_SLOTS:
 	void pasteData();
 	void updateFormula();
+	void setSparkline(const QPixmap&);
+	QPixmap sparkline();
+
+protected:
+	void handleAspectUpdated(const QString& aspectPath, const AbstractAspect*);
 
 private:
 	bool XmlReadInputFilter(XmlStreamReader*);
@@ -218,7 +254,6 @@ private:
 	void handleRowInsertion(int before, int count) override;
 	void handleRowRemoval(int first, int count) override;
 
-	bool m_suppressDataChangedSignal{false};
 	QAction* m_copyDataAction{nullptr};
 	QAction* m_pasteDataAction{nullptr};
 	QActionGroup* m_usedInActionGroup{nullptr};
@@ -226,6 +261,8 @@ private:
 	ColumnPrivate* d;
 	ColumnStringIO* m_string_io;
 
+	QPixmap m_sparkline;
+
 Q_SIGNALS:
 	void requestProjectContextMenu(QMenu*);
 	void formulaChanged(const AbstractColumn*);
@@ -237,6 +274,9 @@ private Q_SLOTS:
 
 	friend class ColumnPrivate;
 	friend class ColumnStringIO;
+	friend class ColumnRemoveRowsCmd;
+	friend class ColumnInsertRowsCmd;
+	friend class Project; // requires handleAspectUpdated()
 };
 
 #endif
