@@ -1,16 +1,18 @@
 /*
-    File                 : DatapickerCurvePrivate.h
-    Project              : LabPlot
-    Description          : Graphic Item for coordinate points of Datapicker
-    --------------------------------------------------------------------
-    SPDX-FileCopyrightText: 2015 Ankit Wagadre <wagadre.ankit@gmail.com>
-    SPDX-FileCopyrightText: 2015-2021 Alexander Semke <alexander.semke@web.de>
-    SPDX-License-Identifier: GPL-2.0-or-later
+	File                 : DatapickerCurvePrivate.h
+	Project              : LabPlot
+	Description          : Graphic Item for coordinate points of Datapicker
+	--------------------------------------------------------------------
+	SPDX-FileCopyrightText: 2015 Ankit Wagadre <wagadre.ankit@gmail.com>
+	SPDX-FileCopyrightText: 2015-2021 Alexander Semke <alexander.semke@web.de>
+	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #ifndef DATAPICKERCURVEPRIVATE_H
 #define DATAPICKERCURVEPRIVATE_H
 
+#include "backend/worksheet/Worksheet.h"
+
 class Symbol;
 class QBrush;
 class QPen;
@@ -24,12 +26,12 @@ public:
 	DatapickerCurve* const q;
 	void retransform();
 
-	Symbol* symbol;
-	DatapickerCurve::Errors curveErrorTypes;
+	Symbol* symbol{nullptr};
+	DatapickerCurve::Errors curveErrorTypes{DatapickerCurve::ErrorType::NoError, DatapickerCurve::ErrorType::NoError};
 	QBrush pointErrorBarBrush;
 	QPen pointErrorBarPen;
-	qreal pointErrorBarSize;
-	bool pointVisibility;
+	qreal pointErrorBarSize{Worksheet::convertToSceneUnits(8, Worksheet::Unit::Point)};
+	bool pointVisibility{true};
 
 	AbstractColumn* posXColumn{nullptr};
 	QString posXColumnPath;
@@ -45,6 +47,8 @@ public:
 	QString plusDeltaYColumnPath;
 	AbstractColumn* minusDeltaYColumn{nullptr};
 	QString minusDeltaYColumnPath;
+
+	friend class DatapickerTest;
 };
 
 #endif
