@@ -20,8 +20,8 @@
 #include "backend/lib/XmlStreamReader.h"
 #include "backend/lib/trace.h"
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
-#include "commonfrontend/spreadsheet/SpreadsheetView.h"
-#include "kdefrontend/spreadsheet/PlotDataDialog.h"
+#include "frontend/spreadsheet/PlotDataDialog.h"
+#include "frontend/spreadsheet/SpreadsheetView.h"
 
 #include <QAction>
 #include <QDateTime>
@@ -50,13 +50,16 @@ LiveDataSource::LiveDataSource(const QString& name, bool loading)
 	: Spreadsheet(name, loading, AspectType::LiveDataSource)
 	, m_updateTimer(new QTimer(this))
 	, m_watchTimer(new QTimer(this)) {
+	setSuppressSetCommentFinalizeImport(true);
 	m_watchTimer->setSingleShot(true);
 	m_watchTimer->setInterval(100); // maximum read frequency is 1/100ms = 10Hz
 
 	// stop reading from the source before removing the child from the project
-	connect(this, &AbstractAspect::childAspectAboutToBeRemoved, [this](const AbstractAspect* aspect) {
-		if (aspect == this)
+	connect(this, &AbstractAspect::aspectAboutToBeRemoved, [this](const AbstractAspect* aspect) {
+		if (aspect == this) {
 			pauseReading();
+			m_device->close();
+		}
 	});
 
 	connect(m_updateTimer, &QTimer::timeout, this, &LiveDataSource::read);
@@ -69,17 +72,19 @@ LiveDataSource::~LiveDataSource() {
 
 	delete m_filter;
 	delete m_fileSystemWatcher;
-	delete m_localSocket;
-	delete m_tcpSocket;
-#ifdef HAVE_QTSERIALPORT
-	delete m_serialPort;
-#endif
+	// Deleting m_device is enough!
+	// 	delete m_localSocket;
+	// 	delete m_tcpSocket;
+	// #ifdef HAVE_QTSERIALPORT
+	// 	delete m_serialPort;
+	// #endif
 	delete m_device;
 }
 
 QWidget* LiveDataSource::view() const {
 	if (!m_partView) {
 		m_view = new SpreadsheetView(const_cast<LiveDataSource*>(this), true);
+		m_view->setSuppressResizeHeader(true);
 		m_partView = m_view;
 	}
 	return m_partView;
@@ -123,7 +128,7 @@ QStringList LiveDataSource::supportedBaudRates() {
  * \brief Updates this data source at this moment
  */
 void LiveDataSource::updateNow() {
-	DEBUG("LiveDataSource::updateNow() update interval = " << m_updateInterval);
+	DEBUG("LiveDataSource::updateNow() update interval = " << m_updateInterval_ms << "ms");
 	if (m_updateType == UpdateType::TimeInterval)
 		m_updateTimer->stop();
 	else
@@ -132,7 +137,7 @@ void LiveDataSource::updateNow() {
 
 	// restart the timer after update
 	if (m_updateType == UpdateType::TimeInterval && !m_paused)
-		m_updateTimer->start(m_updateInterval);
+		m_updateTimer->start(m_updateInterval_ms);
 }
 
 /*!
@@ -187,6 +192,9 @@ AbstractFileFilter::FileType LiveDataSource::fileType() const {
 
 void LiveDataSource::setFilter(AbstractFileFilter* f) {
 	delete m_filter;
+	auto* asciiFilter = dynamic_cast<AsciiFilter*>(f);
+	if (asciiFilter)
+		asciiFilter->setDataSource(this);
 	m_filter = f;
 }
 
@@ -210,14 +218,19 @@ int LiveDataSource::baudRate() const {
  * \brief Sets the source's update interval to \c interval
  * \param interval
  */
-void LiveDataSource::setUpdateInterval(int interval) {
-	m_updateInterval = interval;
+void LiveDataSource::setUpdateInterval(int interval_ms) {
+	m_updateInterval_ms = interval_ms;
 	if (!m_paused)
-		m_updateTimer->start(m_updateInterval);
+		m_updateTimer->start(m_updateInterval_ms);
 }
 
+/*!
+ * \brief LiveDataSource::updateInterval
+ * Get update interval in ms
+ * \return
+ */
 int LiveDataSource::updateInterval() const {
-	return m_updateInterval;
+	return m_updateInterval_ms;
 }
 
 /*!
@@ -238,6 +251,7 @@ int LiveDataSource::keepNValues() const {
  */
 void LiveDataSource::setPort(quint16 port) {
 	m_port = port;
+	initDevice();
 }
 
 void LiveDataSource::setBytesRead(qint64 bytes) {
@@ -352,7 +366,11 @@ LiveDataSource::UpdateType LiveDataSource::updateType() const {
  * \param host
  */
 void LiveDataSource::setHost(const QString& host) {
-	m_host = host.simplified();
+	if (host.compare(QStringLiteral("localhost"), Qt::CaseSensitivity::CaseInsensitive) == 0)
+		m_host = QStringLiteral("127.0.0.1");
+	else
+		m_host = host.simplified();
+	initDevice();
 }
 
 QString LiveDataSource::host() const {
@@ -420,6 +438,7 @@ QIcon LiveDataSource::icon() const {
 	case AbstractFileFilter::FileType::HDF5:
 	case AbstractFileFilter::FileType::NETCDF:
 	case AbstractFileFilter::FileType::VECTOR_BLF:
+	case AbstractFileFilter::FileType::MCAP:
 		break;
 	}
 
@@ -455,6 +474,82 @@ void LiveDataSource::readOnUpdate() {
 	Q_EMIT readOnUpdateCalled();
 }
 
+void LiveDataSource::initDevice() {
+	switch (m_sourceType) {
+	case SourceType::FileOrPipe:
+		delete m_device;
+		m_device = new QFile(m_fileName);
+		break;
+	case SourceType::NetworkTCPSocket:
+		if (!m_tcpSocket)
+			m_tcpSocket = new QTcpSocket(this);
+		m_device = m_tcpSocket;
+		m_tcpSocket->abort();
+		m_tcpSocket->connectToHost(m_host, m_port, QIODevice::ReadOnly);
+
+		connect(m_tcpSocket, &QTcpSocket::readyRead, this, &LiveDataSource::readyRead);
+		connect(m_tcpSocket,
+				static_cast<void (QTcpSocket::*)(QAbstractSocket::SocketError)>(&QTcpSocket::errorOccurred),
+				this,
+				&LiveDataSource::tcpSocketError);
+		break;
+	case SourceType::NetworkUDPSocket: {
+		if (!m_udpSocket)
+			m_udpSocket = new QUdpSocket(this);
+		m_device = m_udpSocket;
+		m_udpSocket->abort();
+		if (m_udpSocket->bind(QHostAddress(m_host), m_port)) {
+			if (m_updateType == UpdateType::NewData)
+				connect(m_udpSocket, &QUdpSocket::readyRead, this, &LiveDataSource::readyRead);
+			m_udpSocket->connectToHost(m_host, 0, QUdpSocket::ReadOnly);
+
+			if (m_udpSocket->waitForConnected()) {
+				// only connect to readyRead when update is on new data
+				connect(m_udpSocket,
+						static_cast<void (QUdpSocket::*)(QAbstractSocket::SocketError)>(&QUdpSocket::errorOccurred),
+						this,
+						&LiveDataSource::tcpSocketError);
+			} else
+				DEBUG("failed to connect to UDP socket - " << STDSTRING(m_udpSocket->errorString()));
+		} else
+			DEBUG("Unable to bind - " << m_udpSocket->errorString().toStdString());
+
+		break;
+	}
+	case SourceType::LocalSocket:
+		if (!m_localSocket)
+			m_localSocket = new QLocalSocket(this);
+		m_device = m_localSocket;
+		m_localSocket->abort();
+		m_localSocket->connectToServer(m_localSocketName, QLocalSocket::ReadOnly);
+
+		connect(m_localSocket, &QLocalSocket::readyRead, this, &LiveDataSource::readyRead);
+		connect(m_localSocket,
+				static_cast<void (QLocalSocket::*)(QLocalSocket::LocalSocketError)>(&QLocalSocket::errorOccurred),
+				this,
+				&LiveDataSource::localSocketError);
+		break;
+	case SourceType::SerialPort:
+#ifdef HAVE_QTSERIALPORT
+		if (!m_serialPort)
+			m_serialPort = new QSerialPort(this);
+		m_device = m_serialPort;
+		DEBUG("	Serial: " << STDSTRING(m_serialPortName) << ", " << m_baudRate);
+		m_serialPort->setBaudRate(m_baudRate);
+		m_serialPort->setPortName(m_serialPortName);
+		// m_serialPort->open(QIODevice::ReadOnly); // Not required
+
+		// only connect to readyRead when update is on new data
+		if (m_updateType == UpdateType::NewData)
+			connect(m_serialPort, &QSerialPort::readyRead, this, &LiveDataSource::readyRead);
+		connect(m_serialPort, &QSerialPort::errorOccurred, this, &LiveDataSource::serialPortError);
+#endif
+		break;
+	case SourceType::MQTT:
+		break;
+	}
+}
+
 /*
  * called periodically or on new data changes (file changed, new data in the socket, etc.)
  */
@@ -466,88 +561,13 @@ void LiveDataSource::read() {
 	if (m_reading)
 		return;
 
+	static bool firstRead = true;
+
 	m_reading = true;
 
 	// initialize the device (file, socket, serial port) when calling this function for the first time
 	if (!m_prepared) {
-		DEBUG("	Preparing device: update type = " << ENUM_TO_STRING(LiveDataSource, UpdateType, m_updateType));
-		switch (m_sourceType) {
-		case SourceType::FileOrPipe:
-			delete m_device;
-			m_device = new QFile(m_fileName);
-			break;
-		case SourceType::NetworkTCPSocket:
-			m_tcpSocket = new QTcpSocket(this);
-			m_device = m_tcpSocket;
-			m_tcpSocket->connectToHost(m_host, m_port, QIODevice::ReadOnly);
-
-			connect(m_tcpSocket, &QTcpSocket::readyRead, this, &LiveDataSource::readyRead);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
-			connect(m_tcpSocket,
-					static_cast<void (QTcpSocket::*)(QAbstractSocket::SocketError)>(&QTcpSocket::errorOccurred),
-					this,
-					&LiveDataSource::tcpSocketError);
-#else
-			connect(m_tcpSocket, static_cast<void (QTcpSocket::*)(QAbstractSocket::SocketError)>(&QTcpSocket::error), this, &LiveDataSource::tcpSocketError);
-#endif
-
-			break;
-		case SourceType::NetworkUDPSocket:
-			m_udpSocket = new QUdpSocket(this);
-			m_device = m_udpSocket;
-			m_udpSocket->bind(QHostAddress(m_host), m_port);
-			m_udpSocket->connectToHost(m_host, 0, QUdpSocket::ReadOnly);
-
-			// only connect to readyRead when update is on new data
-			if (m_updateType == UpdateType::NewData)
-				connect(m_udpSocket, &QUdpSocket::readyRead, this, &LiveDataSource::readyRead);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
-			connect(m_udpSocket,
-					static_cast<void (QUdpSocket::*)(QAbstractSocket::SocketError)>(&QUdpSocket::errorOccurred),
-					this,
-					&LiveDataSource::tcpSocketError);
-#else
-			connect(m_udpSocket, static_cast<void (QUdpSocket::*)(QAbstractSocket::SocketError)>(&QUdpSocket::error), this, &LiveDataSource::tcpSocketError);
-#endif
-
-			break;
-		case SourceType::LocalSocket:
-			m_localSocket = new QLocalSocket(this);
-			m_device = m_localSocket;
-			m_localSocket->connectToServer(m_localSocketName, QLocalSocket::ReadOnly);
-
-			connect(m_localSocket, &QLocalSocket::readyRead, this, &LiveDataSource::readyRead);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
-			connect(m_localSocket,
-					static_cast<void (QLocalSocket::*)(QLocalSocket::LocalSocketError)>(&QLocalSocket::errorOccurred),
-					this,
-					&LiveDataSource::localSocketError);
-#else
-			connect(m_localSocket,
-					static_cast<void (QLocalSocket::*)(QLocalSocket::LocalSocketError)>(&QLocalSocket::error),
-					this,
-					&LiveDataSource::localSocketError);
-#endif
-
-			break;
-		case SourceType::SerialPort:
-#ifdef HAVE_QTSERIALPORT
-			m_serialPort = new QSerialPort(this);
-			m_device = m_serialPort;
-			DEBUG("	Serial: " << STDSTRING(m_serialPortName) << ", " << m_baudRate);
-			m_serialPort->setBaudRate(m_baudRate);
-			m_serialPort->setPortName(m_serialPortName);
-			m_serialPort->open(QIODevice::ReadOnly);
-
-			// only connect to readyRead when update is on new data
-			if (m_updateType == UpdateType::NewData)
-				connect(m_serialPort, &QSerialPort::readyRead, this, &LiveDataSource::readyRead);
-			connect(m_serialPort, &QSerialPort::errorOccurred, this, &LiveDataSource::serialPortError);
-#endif
-			break;
-		case SourceType::MQTT:
-			break;
-		}
+		initDevice();
 		m_prepared = true;
 	}
 
@@ -557,11 +577,17 @@ void LiveDataSource::read() {
 		switch (m_fileType) {
 		case AbstractFileFilter::FileType::Ascii:
 			if (m_readingType == LiveDataSource::ReadingType::WholeFile) {
-				static_cast<AsciiFilter*>(m_filter)->readFromLiveDevice(*m_device, this, 0);
+				static_cast<AsciiFilter*>(m_filter)
+					->readFromDevice(*m_device, AbstractFileFilter::ImportMode::Replace, AbstractFileFilter::ImportMode::Replace, 0, -1, 0);
 			} else {
-				qint64 bytes = static_cast<AsciiFilter*>(m_filter)->readFromLiveDevice(*m_device, this, m_bytesRead);
+				qint64 bytes = static_cast<AsciiFilter*>(m_filter)->readFromDevice(*m_device,
+																				   AbstractFileFilter::ImportMode::Replace,
+																				   AbstractFileFilter::ImportMode::Append,
+																				   m_bytesRead,
+																				   sampleSize(),
+																				   m_keepNValues);
 				m_bytesRead += bytes;
-				DEBUG("Read " << bytes << " bytes, in total: " << m_bytesRead);
+				// DEBUG("Read " << bytes << " bytes, in total: " << m_bytesRead);
 			}
 			break;
 		case AbstractFileFilter::FileType::Binary:
@@ -584,21 +610,16 @@ void LiveDataSource::read() {
 		case AbstractFileFilter::FileType::JSON:
 		case AbstractFileFilter::FileType::READSTAT:
 		case AbstractFileFilter::FileType::MATIO:
+		case AbstractFileFilter::FileType::MCAP:
 			break;
 		}
 		break;
-	case SourceType::NetworkTCPSocket:
-		DEBUG("reading from TCP socket. state before abort = " << m_tcpSocket->state());
-		m_tcpSocket->abort();
-		m_tcpSocket->connectToHost(m_host, m_port, QIODevice::ReadOnly);
-		DEBUG("reading from TCP socket. state after reconnect = " << m_tcpSocket->state());
-		break;
+	case SourceType::NetworkTCPSocket: // fall through
 	case SourceType::NetworkUDPSocket:
-		DEBUG("	Reading from UDP socket. state = " << m_udpSocket->state());
-
 		// reading data here
 		if (m_fileType == AbstractFileFilter::FileType::Ascii)
-			static_cast<AsciiFilter*>(m_filter)->readFromLiveDeviceNotFile(*m_device, this);
+			static_cast<AsciiFilter*>(m_filter)
+				->readFromDevice(*m_device, AbstractFileFilter::ImportMode::Replace, AbstractFileFilter::ImportMode::Append, 0, sampleSize(), m_keepNValues);
 		break;
 	case SourceType::LocalSocket:
 		DEBUG("	Reading from local socket. state before abort = " << m_localSocket->state());
@@ -609,51 +630,33 @@ void LiveDataSource::read() {
 			m_localSocket->waitForReadyRead();
 		DEBUG("	Reading from local socket. state after reconnect = " << m_localSocket->state());
 		break;
-	case SourceType::SerialPort:
+	case SourceType::SerialPort: {
 		DEBUG("	Reading from serial port");
 #ifdef HAVE_QTSERIALPORT
-
 		// reading data here
-		if (m_fileType == AbstractFileFilter::FileType::Ascii)
-			static_cast<AsciiFilter*>(m_filter)->readFromLiveDeviceNotFile(*m_device, this);
+		if (m_fileType == AbstractFileFilter::FileType::Ascii) {
+			if (firstRead)
+				static_cast<AsciiFilter*>(m_filter)->clearLastError();
+			static_cast<AsciiFilter*>(m_filter)->readFromDevice(*m_device,
+																AbstractFileFilter::ImportMode::Replace,
+																AbstractFileFilter::ImportMode::Append,
+																0,
+																sampleSize(),
+																m_keepNValues,
+																firstRead);
+			if (static_cast<AsciiFilter*>(m_filter)->lastError().isEmpty())
+				firstRead = false;
+		}
 #endif
 		break;
+	}
 	case SourceType::MQTT:
 		break;
 	}
 
-	finalizeRead();
-
 	m_reading = false;
 }
 
-/*!
- * notify all affected columns and plots about the changes.
- * this is simililar to \sa Spreadsheet::finalizeImport().
- */
-void LiveDataSource::finalizeRead() {
-	PERFTRACE(QLatin1String("AsciiLiveDataImport, notify affected columns and plots"));
-
-	// determine the dependent plots
-	QVector<CartesianPlot*> plots;
-	const auto& columns = children<Column>();
-	for (auto* column : columns)
-		column->addUsedInPlots(plots);
-
-	// suppress retransform in the dependent plots
-	for (auto* plot : plots)
-		plot->setSuppressRetransform(true);
-
-	for (auto* column : columns)
-		column->setChanged();
-
-	// retransform the dependent plots
-	for (auto* plot : plots) {
-		plot->setSuppressRetransform(false);
-		plot->dataChanged(-1, -1); // TODO: check if all ranges must be updated!
-	}
-}
-
 /*!
  * Slot for the signal that is emitted once every time new data is available for reading from the device (not UDP or Serial).
  * It will only be emitted again once new data is available, such as when a new payload of network data has arrived on the network socket,
@@ -663,8 +666,8 @@ void LiveDataSource::readyRead() {
 	DEBUG(Q_FUNC_INFO << ", update type = " << ENUM_TO_STRING(LiveDataSource, UpdateType, m_updateType));
 	DEBUG("	REMAINING TIME = " << m_updateTimer->remainingTime());
 
-	if (m_fileType == AbstractFileFilter::FileType::Ascii)
-		static_cast<AsciiFilter*>(m_filter)->readFromLiveDeviceNotFile(*m_device, this);
+	// if (m_fileType == AbstractFileFilter::FileType::Ascii)
+	// 	static_cast<AsciiFilter*>(m_filter)->readFromLiveDeviceNotFile(*m_device, this); // TODO: turn on again
 
 	// TODO: not implemented yet
 	//	else if (m_fileType == AbstractFileFilter::FileType::Binary)
@@ -720,42 +723,45 @@ void LiveDataSource::tcpSocketError(QAbstractSocket::SocketError /*socketError*/
 }
 
 #ifdef HAVE_QTSERIALPORT
-void LiveDataSource::serialPortError(QSerialPort::SerialPortError error) {
+QString LiveDataSource::serialPortErrorEnumToString(QSerialPort::SerialPortError error, const QString errorString) {
+	QString msg;
 	switch (error) {
 	case QSerialPort::DeviceNotFoundError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("Failed to open the device."));
+		msg = i18n("Device doesn't exist.");
 		break;
 	case QSerialPort::PermissionError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("Failed to open the device. Please check your permissions on this device."));
+		msg = i18n("User doesn't have enough permissions to open the device.");
 		break;
 	case QSerialPort::OpenError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("Device already opened."));
+		msg = i18n("Device already opened.");
 		break;
 	case QSerialPort::NotOpenError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("The device is not opened."));
+		msg = i18n("Device is not open.");
 		break;
 	case QSerialPort::ReadError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("Failed to read data."));
+		msg = i18n("Failed to read data.");
 		break;
 	case QSerialPort::ResourceError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("Failed to read data. The device is removed."));
+		msg = i18n("Device not available.");
 		break;
 	case QSerialPort::TimeoutError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("The device timed out."));
+		msg = i18n("Timeout occured.");
 		break;
 	case QSerialPort::WriteError:
 	case QSerialPort::UnsupportedOperationError:
-#if QT_VERSION <= QT_VERSION_CHECK(6, 0, 0)
-	case QSerialPort::ParityError:
-	case QSerialPort::FramingError:
-	case QSerialPort::BreakConditionError:
-#endif
 	case QSerialPort::UnknownError:
-		QMessageBox::critical(nullptr, i18n("Serial Port Error"), i18n("The following error occurred: %1.", m_serialPort->errorString()));
+		msg = i18n("The following error occurred: %1.", errorString);
 		break;
 	case QSerialPort::NoError:
 		break;
 	}
+
+	return msg;
+}
+
+void LiveDataSource::serialPortError(QSerialPort::SerialPortError error) {
+	if (error != QSerialPort::SerialPortError::NoError)
+		QMessageBox::critical(nullptr, i18n("Serial Port Error"), serialPortErrorEnumToString(error, m_serialPort->errorString()));
 }
 #endif
 
@@ -813,7 +819,7 @@ void LiveDataSource::save(QXmlStreamWriter* writer) const {
 	writer->writeAttribute(QStringLiteral("keepNValues"), QString::number(m_keepNValues));
 
 	if (m_updateType == UpdateType::TimeInterval)
-		writer->writeAttribute(QStringLiteral("updateInterval"), QString::number(m_updateInterval));
+		writer->writeAttribute(QStringLiteral("updateInterval"), QString::number(m_updateInterval_ms));
 
 	if (m_readingType != ReadingType::TillEnd)
 		writer->writeAttribute(QStringLiteral("sampleSize"), QString::number(m_sampleSize));
@@ -908,7 +914,7 @@ bool LiveDataSource::load(XmlStreamReader* reader, bool preview) {
 				if (str.isEmpty())
 					reader->raiseMissingAttributeWarning(QStringLiteral("updateInterval"));
 				else
-					m_updateInterval = str.toInt();
+					m_updateInterval_ms = str.toInt();
 			}
 
 			if (m_readingType != ReadingType::TillEnd) {
