@@ -111,7 +111,7 @@ void ROOTFilter::save(QXmlStreamWriter* writer) const {
 
 bool ROOTFilter::load(XmlStreamReader* reader) {
 	QString attributeWarning = i18n("Attribute '%1' missing or empty, default value is used");
-	QXmlStreamAttributes attribs = reader->attributes();
+	auto attribs = reader->attributes();
 
 	// read attributes
 	d->currentObject = attribs.value(QStringLiteral("object")).toString();
@@ -158,7 +158,7 @@ ROOTFilterPrivate::ROOTFilterPrivate(ROOTFilter* owner)
 }
 
 ROOTFilterPrivate::FileType ROOTFilterPrivate::currentObjectPosition(const QString& fileName, long int& pos) {
-	QStringList typeobject = currentObject.split(QLatin1Char(':'));
+	auto typeobject = currentObject.split(QLatin1Char(':'));
 	if (typeobject.size() < 2) {
 		q->setLastError(i18n("Unsupported file type, neither histogram no tree."));
 		return FileType::Invalid;
@@ -173,9 +173,9 @@ ROOTFilterPrivate::FileType ROOTFilterPrivate::currentObjectPosition(const QStri
 		return FileType::Invalid;
 
 	typeobject.removeFirst();
-	QStringList path = typeobject.join(QLatin1Char(':')).split(QLatin1Char('/'));
-	ROOTFilter::Directory dir = type == FileType::Hist ? listHistograms(fileName) : listTrees(fileName);
-	const ROOTFilter::Directory* node = &dir;
+	auto path = typeobject.join(QLatin1Char(':')).split(QLatin1Char('/'));
+	const auto& dir = (type == FileType::Hist) ? listHistograms(fileName) : listTrees(fileName);
+	const auto* node = &dir;
 	while (path.size() > 1) {
 		bool next = false;
 		for (const auto& child : node->children) {
@@ -219,9 +219,8 @@ void ROOTFilterPrivate::readDataFromFile(const QString& fileName, AbstractDataSo
 		DEBUG("first/last = " << first << " " << last)
 
 		QStringList headers;
-		for (const auto& l : columns) {
+		for (const auto& l : columns)
 			headers << l.last();
-		}
 
 		std::vector<void*> dataContainer;
 		bool ok = false;
@@ -238,7 +237,7 @@ void ROOTFilterPrivate::readDataFromFile(const QString& fileName, AbstractDataSo
 		}
 
 		// read data
-		DEBUG("	reading " << last - first + 1 << " lines");
+		DEBUG(Q_FUNC_INFO << ", reading " << last - first + 1 << " lines");
 
 		int c = 0;
 		auto* spreadsheet = dynamic_cast<Spreadsheet*>(dataSource);
@@ -278,7 +277,7 @@ void ROOTFilterPrivate::readDataFromFile(const QString& fileName, AbstractDataSo
 		int first = std::max(std::abs(startRow), 0);
 		int last = std::max(first - 1, std::min(endRow, nentries - 1));
 
-		DEBUG("first/last = " << first << " " << last << ", nentries = " << nentries)
+		DEBUG(Q_FUNC_INFO << ", first/last = " << first << " " << last << ", nentries = " << nentries)
 
 		QStringList headers;
 		for (const auto& l : columns) {
@@ -322,7 +321,7 @@ void ROOTFilterPrivate::readDataFromFile(const QString& fileName, AbstractDataSo
 			} else if (l.count() > 1)
 				leaf = l.at(1);
 
-			QVector<double>& container = *static_cast<QVector<double>*>(dataContainer[c++]);
+			auto& container = *static_cast<QVector<double>*>(dataContainer[c++]);
 			auto data = readTree(pos, l.first(), leaf, (int)element, last);
 			// QDEBUG("DATA = " << data)
 			for (int i = first; i <= last; ++i)
@@ -343,7 +342,7 @@ ROOTFilter::Directory ROOTFilterPrivate::listContent(const std::map<long int, RO
 		if (!path.second.content.empty()) {
 			QStack<decltype(filledDirs)::key_type> addpath;
 			auto pos = &path;
-			ROOTFilter::Directory* currentdir = &dirs;
+			auto* currentdir = &dirs;
 			while (true) {
 				auto it = filledDirs.find(pos);
 				if (it != filledDirs.end()) {
@@ -381,14 +380,14 @@ ROOTFilter::Directory ROOTFilterPrivate::listHistograms(const QString& fileName)
 	if (setFile(fileName))
 		return listContent(currentROOTData->listHistograms(), &ROOTData::histogramName);
 	else
-		return ROOTFilter::Directory{};
+		return {};
 }
 
 ROOTFilter::Directory ROOTFilterPrivate::listTrees(const QString& fileName) {
 	if (setFile(fileName))
 		return listContent(currentROOTData->listTrees(), &ROOTData::treeName);
 	else
-		return ROOTFilter::Directory{};
+		return {};
 }
 
 QVector<QStringList> ROOTFilterPrivate::listLeaves(const QString& fileName, quint64 pos) {
@@ -408,7 +407,7 @@ QVector<QStringList> ROOTFilterPrivate::listLeaves(const QString& fileName, quin
 }
 
 QVector<QStringList> ROOTFilterPrivate::previewCurrentObject(const QString& fileName, int first, int last) {
-	DEBUG(Q_FUNC_INFO);
+	DEBUG(Q_FUNC_INFO << ", first/last = " << first << " " << last);
 
 	long int pos = 0;
 	auto type = currentObjectPosition(fileName, pos);
@@ -422,7 +421,7 @@ QVector<QStringList> ROOTFilterPrivate::previewCurrentObject(const QString& file
 		last = std::min(nbins - 1, last);
 
 		QVector<QStringList> preview(std::max(last - first + 2, 1));
-		DEBUG("	reading " << preview.size() - 1 << " lines");
+		DEBUG(Q_FUNC_INFO << ", reading " << preview.size() - 1 << " lines");
 
 		// set headers
 		for (const auto& l : columns) {
@@ -453,7 +452,7 @@ QVector<QStringList> ROOTFilterPrivate::previewCurrentObject(const QString& file
 		last = std::min(last, currentROOTData->treeEntries(pos) - 1);
 
 		QVector<QStringList> preview(std::max(last - first + 2, 1));
-		DEBUG("	reading " << preview.size() - 1 << " lines");
+		DEBUG(Q_FUNC_INFO << ", TREE: reading " << preview.size() - 1 << " lines");
 
 		// read data leaf by leaf and set headers
 		for (const auto& l : columns) {
@@ -485,6 +484,7 @@ QVector<QStringList> ROOTFilterPrivate::previewCurrentObject(const QString& file
 }
 
 int ROOTFilterPrivate::rowsInCurrentObject(const QString& fileName) {
+	DEBUG(Q_FUNC_INFO)
 	long int pos = 0;
 	auto type = currentObjectPosition(fileName, pos);
 	if (pos == 0)
@@ -526,7 +526,7 @@ std::vector<ROOTData::BinPars> ROOTFilterPrivate::readHistogram(quint64 pos) {
 }
 
 std::vector<double> ROOTFilterPrivate::readTree(quint64 pos, const QString& branchName, const QString& leafName, int element, int last) {
-	// QDEBUG("branch/leaf name =" << branchName << " " << leafName << ", element/last =" << element << " " << last)
+	DEBUG("branch/leaf name =" << branchName.toStdString() << " " << leafName.toStdString() << ", element/last =" << element << " " << last)
 	return currentROOTData->listEntries<double>(pos, branchName.toStdString(), leafName.toStdString(), element, last + 1);
 }
 
@@ -634,6 +634,7 @@ using namespace ROOTDataHelpers;
 
 ROOTData::ROOTData(const std::string& filename)
 	: filename(filename) {
+	DEBUG(Q_FUNC_INFO << ", filename = " << filename)
 	// The file structure is described in root/io/io/src/TFile.cxx
 	std::ifstream is(filename, std::ifstream::binary);
 	std::string root(4, 0);
@@ -681,7 +682,7 @@ ROOTData::ROOTData(const std::string& filename)
 		std::string title(read<unsigned char>(is), 0);
 		is.read(&title[0], title.size());
 
-		ContentType type = ContentType::Invalid;
+		auto type = ContentType::Invalid;
 		if (cname.size() == 4 && cname.substr(0, 3) == "TH1") {
 			type = histType(cname[3]);
 		} else if (cname == "TTree")
@@ -747,6 +748,7 @@ ROOTData::ROOTData(const std::string& filename)
 				break;
 			case ContentType::Tree:
 			case ContentType::NTuple: {
+				DEBUG(Q_FUNC_INFO << ", treekeys ..")
 				auto it = treedirs.find(pseek);
 				if (it == treedirs.end())
 					it = treedirs.begin();
@@ -866,7 +868,7 @@ ROOTData::ROOTData(const std::string& filename)
 }
 
 void ROOTData::readNBins(ROOTData::KeyBuffer& kbuffer) {
-	std::string buffer = data(kbuffer);
+	auto buffer = data(kbuffer);
 	if (!buffer.empty()) {
 		auto search = streamerInfo.find("TH1");
 		if (search == streamerInfo.end())
@@ -995,10 +997,12 @@ std::string ROOTData::treeName(long int pos) {
 }
 
 int ROOTData::treeEntries(long int pos) {
+	DEBUG(Q_FUNC_INFO << ", pos = " << pos)
 	auto it = treekeys.find(pos);
-	if (it != treekeys.end())
+	if (it != treekeys.end()) {
+		DEBUG(Q_FUNC_INFO << ", rows = " << it->second.nrows)
 		return it->second.nrows;
-	else
+	} else
 		return 0;
 }
 
@@ -1010,7 +1014,7 @@ std::vector<ROOTData::LeafInfo> ROOTData::listLeaves(long int pos) const {
 		return leaves;
 
 	std::ifstream is(filename, std::ifstream::binary);
-	std::string datastring = data(it->second, is);
+	auto datastring = data(it->second, is);
 	if (datastring.empty())
 		return leaves;
 
@@ -1041,7 +1045,7 @@ std::vector<ROOTData::LeafInfo> ROOTData::listLeaves(long int pos) const {
 	const size_t lowb = read<int>(buf);
 	std::map<size_t, std::string> tags;
 	for (size_t i = 0; i < nbranches; ++i) {
-		std::string clname = readObject(buf, buf0, tags);
+		auto clname = readObject(buf, buf0, tags);
 		size_t count;
 		Version(buf, count); // TBranch or TBranchElement
 		char* const nbuf = buf + count;
@@ -1052,7 +1056,7 @@ std::vector<ROOTData::LeafInfo> ROOTData::listLeaves(long int pos) const {
 			advanceTo(buf, streamerTBranch, std::string(), "TNamed", counts);
 			Version(buf); // TNamed
 			SkipObject(buf);
-			const std::string branch = String(buf);
+			const auto branch = String(buf);
 			String(buf);
 			// TODO add reading of nested branches (fBranches)
 			advanceTo(buf, streamerTBranch, "TNamed", "fLeaves", counts);
@@ -1064,14 +1068,14 @@ std::vector<ROOTData::LeafInfo> ROOTData::listLeaves(long int pos) const {
 			const size_t nleaves = read<int>(buf);
 			const size_t lowb = read<int>(buf);
 			for (size_t i = 0; i < nleaves; ++i) {
-				std::string clname = readObject(buf, buf0, tags);
+				auto clname = readObject(buf, buf0, tags);
 				Version(buf, count); // TLeaf(D/F/B/S/I/L/C/O)
 				char* nbuf = buf + count;
 				if (i >= lowb && clname.size() == 6 && clname.compare(0, 5, "TLeaf") == 0) {
 					Version(buf); // TLeaf
 					Version(buf); // TNamed
 					SkipObject(buf);
-					const std::string leafname = String(buf);
+					const auto leafname = String(buf);
 					String(buf); // title
 					size_t elements = read<int>(buf);
 					int bytes = read<int>(buf);
@@ -1100,7 +1104,7 @@ ROOTData::listEntries(long int pos, const std::string& branchname, const std::st
 		return entries;
 
 	std::ifstream is(filename, std::ifstream::binary);
-	std::string datastring = data(it->second, is);
+	auto datastring = data(it->second, is);
 	if (datastring.empty())
 		return entries;
 
@@ -1133,7 +1137,7 @@ ROOTData::listEntries(long int pos, const std::string& branchname, const std::st
 	const size_t lowb = read<int>(buf);
 	std::map<size_t, std::string> tags;
 	for (size_t i = 0; i < nbranches; ++i) {
-		std::string clname = readObject(buf, buf0, tags);
+		auto clname = readObject(buf, buf0, tags);
 		size_t count;
 		Version(buf, count); // TBranch or TBranchElement
 		char* const nbuf = buf + count;
@@ -1143,7 +1147,7 @@ ROOTData::listEntries(long int pos, const std::string& branchname, const std::st
 			}
 			Version(buf); // TNamed
 			SkipObject(buf);
-			const std::string currentbranch = String(buf);
+			const auto currentbranch = String(buf);
 			String(buf);
 
 			advanceTo(buf, streamerTBranch, "TNamed", "fWriteBasket", counts);
@@ -1159,9 +1163,9 @@ ROOTData::listEntries(long int pos, const std::string& branchname, const std::st
 			const size_t lowb = read<int>(buf);
 			int leafoffset = 0, leafcount = 0, leafcontent = 0, leafsize = 0;
 			bool leafsign = false;
-			ContentType leaftype = ContentType::Invalid;
+			auto leaftype = ContentType::Invalid;
 			for (size_t i = 0; i < nleaves; ++i) {
-				std::string clname = readObject(buf, buf0, tags);
+				auto clname = readObject(buf, buf0, tags);
 				Version(buf, count); // TLeaf(D/F/L/I/S/B/O/C/Element)
 				char* nbuf = buf + count;
 				if (currentbranch == branchname) {
@@ -1218,7 +1222,7 @@ ROOTData::listEntries(long int pos, const std::string& branchname, const std::st
 				long int pos = read<long int>(buf);
 				auto it = basketkeys.find(pos);
 				if (it != basketkeys.end()) {
-					std::string basketbuffer = data(it->second);
+					auto basketbuffer = data(it->second);
 					if (!basketbuffer.empty()) {
 						char* bbuf = &basketbuffer[0];
 						char* const bufend = bbuf + basketbuffer.size();
@@ -1341,7 +1345,7 @@ std::string ROOTData::data(const ROOTData::KeyBuffer& buffer, std::ifstream& is)
 
 void ROOTData::readStreamerInfo(const ROOTData::KeyBuffer& buffer) {
 	std::ifstream is(filename, std::ifstream::binary);
-	std::string datastring = data(buffer, is);
+	auto datastring = data(buffer, is);
 	if (!datastring.empty()) {
 		char* buf = &datastring[0];
 		char* const buf0 = buf - buffer.keylength;
@@ -1351,7 +1355,7 @@ void ROOTData::readStreamerInfo(const ROOTData::KeyBuffer& buffer) {
 		const int nobj = read<int>(buf);
 		std::map<size_t, std::string> tags;
 		for (int i = 0; i < nobj; ++i) {
-			std::string clname = readObject(buf, buf0, tags);
+			auto clname = readObject(buf, buf0, tags);
 			size_t count;
 			Version(buf, count);
 			char* const nbuf = buf + count;
@@ -1374,7 +1378,7 @@ void ROOTData::readStreamerInfo(const ROOTData::KeyBuffer& buffer) {
 				const int nobj = read<int>(buf);
 				const int lowb = read<int>(buf);
 				for (int i = 0; i < nobj; ++i) {
-					std::string clname = readObject(buf, buf0, tags);
+					auto clname = readObject(buf, buf0, tags);
 					Version(buf, count);
 					char* const nbuf = buf + count;
 
@@ -1490,7 +1494,7 @@ bool ROOTData::advanceTo(char*& buf,
 // needs to be after ROOTDataHelpers namespace declaration
 
 QString ROOTFilter::fileInfoString(const QString& fileName) {
-	DEBUG("ROOTFilter::fileInfoString()");
+	DEBUG(Q_FUNC_INFO);
 	QString info;
 
 	// The file structure is described in root/io/io/src/TFile.cxx
@@ -1498,7 +1502,7 @@ QString ROOTFilter::fileInfoString(const QString& fileName) {
 	std::string root(4, 0);
 	is.read(const_cast<char*>(root.data()), 4);
 	if (root != "root") {
-		DEBUG("	Not a ROOT file. root = " << root);
+		DEBUG(Q_FUNC_INFO << ", Not a ROOT file. root = " << root);
 		return i18n("Not a ROOT file");
 	}
 
