@@ -1,15 +1,14 @@
 /*
-    File                 : MatrixModel.cpp
-    Project              : LabPlot
-    Description          : Matrix data model
-    --------------------------------------------------------------------
-    SPDX-FileCopyrightText: 2015-2016 Alexander Semke <alexander.semke@web.de>
-    SPDX-FileCopyrightText: 2008-2009 Tilman Benkert <thzs@gmx.net>
-    SPDX-FileCopyrightText: 2018-2020 Stefan Gerlach <stefan.gerlach@uni.kn>
-    SPDX-License-Identifier: GPL-2.0-or-later
+	File                 : MatrixModel.cpp
+	Project              : LabPlot
+	Description          : Matrix data model
+	--------------------------------------------------------------------
+	SPDX-FileCopyrightText: 2015-2016 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2008-2009 Tilman Benkert <thzs@gmx.net>
+	SPDX-FileCopyrightText: 2018-2020 Stefan Gerlach <stefan.gerlach@uni.kn>
+	SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-
 #include "backend/matrix/MatrixModel.h"
 #include "backend/matrix/Matrix.h"
 
@@ -27,7 +26,9 @@
 
 	\ingroup backend
 */
-MatrixModel::MatrixModel(Matrix* matrix) : QAbstractItemModel(nullptr), m_matrix(matrix) {
+MatrixModel::MatrixModel(Matrix* matrix)
+	: QAbstractItemModel(nullptr)
+	, m_matrix(matrix) {
 	connect(m_matrix, &Matrix::columnsAboutToBeInserted, this, &MatrixModel::handleColumnsAboutToBeInserted);
 	connect(m_matrix, &Matrix::columnsInserted, this, &MatrixModel::handleColumnsInserted);
 	connect(m_matrix, &Matrix::columnsAboutToBeRemoved, this, &MatrixModel::handleColumnsAboutToBeRemoved);
@@ -58,40 +59,40 @@ Qt::ItemFlags MatrixModel::flags(const QModelIndex& index) const {
 }
 
 QVariant MatrixModel::data(const QModelIndex& index, int role) const {
-	if ( !index.isValid() )
+	if (!index.isValid())
 		return {};
 
 	int row = index.row();
 	int col = index.column();
 
 	switch (role) {
-		case Qt::ToolTipRole:
-		case Qt::EditRole:
-		case Qt::DisplayRole: {
-			auto mode = m_matrix->mode();
-			//DEBUG("MatrixModel::data() DisplayRole, mode = " << mode);
-			switch (mode) {
-			case AbstractColumn::ColumnMode::Double:
-				return {m_matrix->text<double>(row, col)};
-			case AbstractColumn::ColumnMode::Integer:
-				return {m_matrix->text<int>(row, col)};
-			case AbstractColumn::ColumnMode::BigInt:
-				return {m_matrix->text<qint64>(row, col)};
-			case AbstractColumn::ColumnMode::DateTime:
-			case AbstractColumn::ColumnMode::Month:
-			case AbstractColumn::ColumnMode::Day:
-				return {m_matrix->text<QDateTime>(row, col)};
-			case AbstractColumn::ColumnMode::Text:	// should not happen
-				return {m_matrix->text<QString>(row, col)};
-			}
-			break;
+	case Qt::ToolTipRole:
+	case Qt::EditRole:
+	case Qt::DisplayRole: {
+		auto mode = m_matrix->mode();
+		// DEBUG(Q_FUNC_INFO << ", DisplayRole, mode = " << mode);
+		switch (mode) {
+		case AbstractColumn::ColumnMode::Double:
+			return {m_matrix->text<double>(row, col)};
+		case AbstractColumn::ColumnMode::Integer:
+			return {m_matrix->text<int>(row, col)};
+		case AbstractColumn::ColumnMode::BigInt:
+			return {m_matrix->text<qint64>(row, col)};
+		case AbstractColumn::ColumnMode::DateTime:
+		case AbstractColumn::ColumnMode::Month:
+		case AbstractColumn::ColumnMode::Day:
+			return {m_matrix->text<QDateTime>(row, col)};
+		case AbstractColumn::ColumnMode::Text: // should not happen
+			return {m_matrix->text<QString>(row, col)};
 		}
-		case Qt::BackgroundRole:
-			//use bluish background color to distinguish Matrix from Spreadsheet
-			return {QColor(192,255,255)};
-		case Qt::ForegroundRole:
-			//ignore current theme settings and always use black foreground color so Matrix is usable with dark themes, too.
-			return {QColor(Qt::black)};
+		break;
+	}
+	case Qt::BackgroundRole:
+		// use bluish background color to distinguish Matrix from Spreadsheet
+		return {QColor(192, 255, 255)};
+	case Qt::ForegroundRole:
+		// ignore current theme settings and always use black foreground color so Matrix is usable with dark themes, too.
+		return {QColor(Qt::black)};
 	}
 
 	return {};
@@ -100,65 +101,61 @@ QVariant MatrixModel::data(const QModelIndex& index, int role) const {
 QVariant MatrixModel::headerData(int section, Qt::Orientation orientation, int role) const {
 	QString result;
 	auto headerFormat = m_matrix->headerFormat();
-	SET_NUMBER_LOCALE
+	const auto numberLocale = QLocale();
 	switch (orientation) {
-		case Qt::Horizontal:
-			switch (role) {
-				case Qt::DisplayRole:
-				case Qt::ToolTipRole:
-					if (headerFormat == Matrix::HeaderFormat::HeaderRowsColumns) {
-						result = QString::number(section+1);
-					} else if (headerFormat == Matrix::HeaderFormat::HeaderValues) {
-						double diff = m_matrix->xEnd() - m_matrix->xStart();
-						double step = 0.0;
-						if (m_matrix->columnCount() > 1)
-							step = diff/double(m_matrix->columnCount()-1);
-						result = numberLocale.toString(m_matrix->xStart()+double(section)*step,
-								m_matrix->numericFormat(), m_matrix->precision());
-					} else {
-						result = QString::number(section+1) + QLatin1String(" (");
-						double diff = m_matrix->xEnd() - m_matrix->xStart();
-						double step = 0.0;
-						if (m_matrix->columnCount() > 1)
-							step = diff/double(m_matrix->columnCount()-1);
-						result += numberLocale.toString(m_matrix->xStart()+double(section)*step,
-								m_matrix->numericFormat(), m_matrix->precision());
+	case Qt::Horizontal:
+		switch (role) {
+		case Qt::DisplayRole:
+		case Qt::ToolTipRole:
+			if (headerFormat == Matrix::HeaderFormat::HeaderRowsColumns) {
+				result = QString::number(section + 1);
+			} else if (headerFormat == Matrix::HeaderFormat::HeaderValues) {
+				double diff = m_matrix->xEnd() - m_matrix->xStart();
+				double step = 0.0;
+				if (m_matrix->columnCount() > 1)
+					step = diff / double(m_matrix->columnCount() - 1);
+				result = numberLocale.toString(m_matrix->xStart() + double(section) * step, m_matrix->numericFormat(), m_matrix->precision());
+			} else {
+				result = QString::number(section + 1) + QLatin1String(" (");
+				double diff = m_matrix->xEnd() - m_matrix->xStart();
+				double step = 0.0;
+				if (m_matrix->columnCount() > 1)
+					step = diff / double(m_matrix->columnCount() - 1);
+				result += numberLocale.toString(m_matrix->xStart() + double(section) * step, m_matrix->numericFormat(), m_matrix->precision());
 
-						result += ')';
-					}
-					return {result};
+				result += QLatin1Char(')');
 			}
-			break;
-		case Qt::Vertical:
-			switch (role) {
-				case Qt::DisplayRole:
-				case Qt::ToolTipRole:
-					if (headerFormat == Matrix::HeaderFormat::HeaderRowsColumns) {
-						result = QString::number(section+1);
-					} else if (headerFormat == Matrix::HeaderFormat::HeaderValues) {
-						double diff = m_matrix->yEnd() - m_matrix->yStart();
-						double step = 0.0;
-						if (m_matrix->rowCount() > 1)
-							step = diff/double(m_matrix->rowCount()-1);
-						// TODO: implement decent double == 0 check
-// 						if (diff < 1e-10)
-// 							result += numberLocale.toString(m_matrix->yStart(),
-// 									m_matrix->numericFormat(), m_matrix->displayedDigits());
-						result += numberLocale.toString(m_matrix->yStart()+double(section)*step,
-								m_matrix->numericFormat(), m_matrix->precision());
-					} else {
-						result = QString::number(section+1) + QString(" (");
-						double diff = m_matrix->yEnd() - m_matrix->yStart();
-						double step = 0.0;
-						if (m_matrix->rowCount() > 1)
-							step = diff/double(m_matrix->rowCount()-1);
+			return {result};
+		}
+		break;
+	case Qt::Vertical:
+		switch (role) {
+		case Qt::DisplayRole:
+		case Qt::ToolTipRole:
+			if (headerFormat == Matrix::HeaderFormat::HeaderRowsColumns) {
+				result = QString::number(section + 1);
+			} else if (headerFormat == Matrix::HeaderFormat::HeaderValues) {
+				double diff = m_matrix->yEnd() - m_matrix->yStart();
+				double step = 0.0;
+				if (m_matrix->rowCount() > 1)
+					step = diff / double(m_matrix->rowCount() - 1);
+				// TODO: implement decent double == 0 check
+				// 						if (diff < 1e-10)
+				// 							result += numberLocale.toString(m_matrix->yStart(),
+				// 									m_matrix->numericFormat(), m_matrix->displayedDigits());
+				result += numberLocale.toString(m_matrix->yStart() + double(section) * step, m_matrix->numericFormat(), m_matrix->precision());
+			} else {
+				result = QString::number(section + 1) + QStringLiteral(" (");
+				double diff = m_matrix->yEnd() - m_matrix->yStart();
+				double step = 0.0;
+				if (m_matrix->rowCount() > 1)
+					step = diff / double(m_matrix->rowCount() - 1);
 
-						result += numberLocale.toString(m_matrix->yStart()+double(section)*step,
-									m_matrix->numericFormat(), m_matrix->precision());
-						result += ')';
-					}
-					return {result};
+				result += numberLocale.toString(m_matrix->yStart() + double(section) * step, m_matrix->numericFormat(), m_matrix->precision());
+				result += QLatin1Char(')');
 			}
+			return {result};
+		}
 	}
 	return {};
 }
@@ -181,7 +178,7 @@ bool MatrixModel::setData(const QModelIndex& index, const QVariant& value, int r
 	if (role == Qt::EditRole) {
 		const auto mode = m_matrix->mode();
 		switch (mode) {
-		case AbstractColumn::ColumnMode::Double: 
+		case AbstractColumn::ColumnMode::Double:
 			m_matrix->setCell(row, column, value.toDouble());
 			break;
 		case AbstractColumn::ColumnMode::Integer:
@@ -193,16 +190,17 @@ bool MatrixModel::setData(const QModelIndex& index, const QVariant& value, int r
 		case AbstractColumn::ColumnMode::DateTime:
 		case AbstractColumn::ColumnMode::Month:
 		case AbstractColumn::ColumnMode::Day:
-			DEBUG("	WARNING: DateTime format not supported yet");	// should not happen
-			//TODO: m_matrix->setCell(row, column, value.toDateTime());
+			DEBUG("	WARNING: DateTime format not supported yet"); // should not happen
+			// TODO: m_matrix->setCell(row, column, value.toDateTime());
 			break;
 		case AbstractColumn::ColumnMode::Text:
-			DEBUG("	WARNING: Text format not supported yet");	// should not happen
+			DEBUG("	WARNING: Text format not supported yet"); // should not happen
 			m_matrix->setCell(row, column, value.toString());
 			break;
 		}
 
-		if (!m_suppressDataChangedSignal) Q_EMIT changed();
+		if (!m_suppressDataChangedSignal)
+			Q_EMIT changed();
 		return true;
 	}
 	return false;
@@ -222,52 +220,57 @@ void MatrixModel::updateHeader() {
 }
 
 void MatrixModel::handleColumnsAboutToBeInserted(int before, int count) {
-	beginInsertColumns(QModelIndex(), before, before+count-1);
+	beginInsertColumns(QModelIndex(), before, before + count - 1);
 }
 
 void MatrixModel::handleColumnsInserted(int /*first*/, int /*count*/) {
 	endInsertColumns();
-	if (!m_suppressDataChangedSignal) Q_EMIT changed();
+	if (!m_suppressDataChangedSignal)
+		Q_EMIT changed();
 }
 
 void MatrixModel::handleColumnsAboutToBeRemoved(int first, int count) {
-	beginRemoveColumns(QModelIndex(), first, first+count-1);
+	beginRemoveColumns(QModelIndex(), first, first + count - 1);
 }
 
 void MatrixModel::handleColumnsRemoved(int /*first*/, int /*count*/) {
 	endRemoveColumns();
-	if (!m_suppressDataChangedSignal) Q_EMIT changed();
+	if (!m_suppressDataChangedSignal)
+		Q_EMIT changed();
 }
 
 void MatrixModel::handleRowsAboutToBeInserted(int before, int count) {
-	beginInsertRows(QModelIndex(), before, before+count-1);
+	beginInsertRows(QModelIndex(), before, before + count - 1);
 }
 
 void MatrixModel::handleRowsInserted(int /*first*/, int /*count*/) {
 	endInsertRows();
-	if (!m_suppressDataChangedSignal) Q_EMIT changed();
+	if (!m_suppressDataChangedSignal)
+		Q_EMIT changed();
 }
 
 void MatrixModel::handleRowsAboutToBeRemoved(int first, int count) {
-	beginRemoveRows(QModelIndex(), first, first+count-1);
+	beginRemoveRows(QModelIndex(), first, first + count - 1);
 }
 
 void MatrixModel::handleRowsRemoved(int /*first*/, int /*count*/) {
 	endRemoveRows();
-	if (!m_suppressDataChangedSignal) Q_EMIT changed();
+	if (!m_suppressDataChangedSignal)
+		Q_EMIT changed();
 }
 
 void MatrixModel::handleDataChanged(int top, int left, int bottom, int right) {
 	Q_EMIT dataChanged(index(top, left), index(bottom, right));
-	if (!m_suppressDataChangedSignal) Q_EMIT changed();
+	if (!m_suppressDataChangedSignal)
+		Q_EMIT changed();
 }
 
 void MatrixModel::handleCoordinatesChanged() {
-	Q_EMIT headerDataChanged(Qt::Horizontal, 0, columnCount()-1);
-	Q_EMIT headerDataChanged(Qt::Vertical, 0, rowCount()-1);
+	Q_EMIT headerDataChanged(Qt::Horizontal, 0, columnCount() - 1);
+	Q_EMIT headerDataChanged(Qt::Vertical, 0, rowCount() - 1);
 }
 
 void MatrixModel::handleFormatChanged() {
 	handleCoordinatesChanged();
-	handleDataChanged(0, 0, rowCount()-1, columnCount()-1);
+	handleDataChanged(0, 0, rowCount() - 1, columnCount() - 1);
 }
