@@ -68,7 +68,7 @@ void Background::init(const KConfigGroup& group) {
 		d->position = (Position)group.readEntry(d->prefix + QStringLiteral("Position"), static_cast<int>(Position::No));
 }
 
-void Background::draw(QPainter* painter, const QPolygonF& polygon) const {
+void Background::draw(QPainter* painter, const QPolygonF& polygon, double radius) const {
 	Q_D(const Background);
 	const QRectF& rect = polygon.boundingRect();
 
@@ -159,7 +159,10 @@ void Background::draw(QPainter* painter, const QPolygonF& polygon) const {
 	painter->setOpacity(d->opacity);
 	painter->setPen(Qt::NoPen);
 
-	painter->drawPolygon(polygon);
+	if (qFuzzyIsNull(radius))
+		painter->drawPolygon(polygon);
+	else
+		painter->drawRoundedRect(rect, radius, radius);
 }
 
 // ##############################################################################
