@@ -126,7 +126,7 @@ public:
 	QPainterPath minorGridPath;
 
 	QVector<QPointF> majorTickPoints; //!< position of the major ticks  on the axis.
-	QVector<QPointF> minorTickPoints; //!< position of the major ticks  on the axis.
+	QVector<QPointF> minorTickPoints; //!< position of the minor ticks  on the axis.
 	QVector<QPointF> tickLabelPoints; //!< position of the major tick labels (left lower edge of label's bounding rect)
 	QVector<double> tickLabelValues; //!< major tick labels values
 	QVector<QString> tickLabelValuesString; //!< major tick labels used when a custom text column is selected
