@@ -3,20 +3,18 @@
 	Project              : LabPlot
 	Description          : Cartesian coordinate system for plots.
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2012-2016 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2012-2025 Alexander Semke <alexander.semke@web.de>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
-
 #include "backend/worksheet/plots/cartesian/CartesianCoordinateSystem.h"
 #include "backend/lib/macros.h"
 #include "backend/worksheet/plots/cartesian/CartesianCoordinateSystemPrivate.h"
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
+#include <KLocalizedString>
 
 #include "backend/nsl/nsl_math.h"
 
-using Dimension = CartesianCoordinateSystem::Dimension;
-
 /* ============================================================================ */
 /* ========================= coordinate system ================================ */
 /* ============================================================================ */
@@ -47,6 +45,9 @@ QString CartesianCoordinateSystem::dimensionToString(Dimension dim) {
 
 QString CartesianCoordinateSystem::info() const {
 	DEBUG(Q_FUNC_INFO)
+	if (!name().isEmpty())
+		return name();
+
 	if (d->plot)
 		return QString(QLatin1String("x = ") + d->plot->range(Dimension::X, d->xIndex).toString() + QLatin1String(", y = ")
 					   + d->plot->range(Dimension::Y, d->yIndex).toString());
