@@ -19,7 +19,12 @@ class SymbolPrivate;
 class KConfigGroup;
 class QPainter;
 
+#ifdef SDK
+#include "labplot_export.h"
+class LABPLOT_EXPORT Symbol : public AbstractAspect {
+#else
 class Symbol : public AbstractAspect {
+#endif
 	Q_OBJECT
 
 public:
@@ -98,8 +103,8 @@ public:
 	~Symbol() override;
 	void init(const KConfigGroup&);
 
-	void draw(QPainter*, QPointF);
-	void draw(QPainter*, const QVector<QPointF>&);
+	void draw(QPainter*, QPointF) const;
+	void draw(QPainter*, const QVector<QPointF>&) const;
 
 	void save(QXmlStreamWriter*) const override;
 	bool load(XmlStreamReader*, bool preview) override;
