@@ -3,7 +3,7 @@
 	Project              : LabPlot
 	Description          : Base class for all analysis curves
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2017-2021 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2017-2024 Alexander Semke <alexander.semke@web.de>
 	SPDX-FileCopyrightText: 2018-2022 Stefan Gerlach <stefan.gerlach@uni.kn>
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -15,12 +15,17 @@
 
 class XYAnalysisCurvePrivate;
 
+#ifdef SDK
+#include "labplot_export.h"
+class LABPLOT_EXPORT XYAnalysisCurve : public XYCurve {
+#else
 class XYAnalysisCurve : public XYCurve {
+#endif
 	Q_OBJECT
-	Q_ENUMS(DataSourceType)
 
 public:
 	enum class DataSourceType { Spreadsheet, Curve, Histogram };
+	Q_ENUM(DataSourceType)
 	enum class AnalysisAction {
 		DataReduction,
 		Differentiation,
@@ -40,9 +45,11 @@ public:
 		FitCustom,
 		FourierFilter
 	};
+	Q_ENUM(AnalysisAction)
 
 	struct Result {
-		Result(){};
+		Result() {
+		}
 
 		bool available{false};
 		bool valid{false};
@@ -60,17 +67,18 @@ public:
 						 double xMax,
 						 bool avgUniqueX = false);
 
-	virtual void recalculate() = 0;
+	void recalculate();
 	bool resultAvailable() const;
 	virtual const Result& result() const = 0;
-	bool usingColumn(const Column*) const override;
+	bool usingColumn(const AbstractColumn*, bool indirect = true) const override;
+	virtual QVector<const Plot*> dependingPlots() const;
 
 	void save(QXmlStreamWriter*) const override;
 	bool load(XmlStreamReader*, bool preview) override;
 
 	BASIC_D_ACCESSOR_DECL(DataSourceType, dataSourceType, DataSourceType)
 	POINTER_D_ACCESSOR_DECL(const XYCurve, dataSourceCurve, DataSourceCurve)
-	const QString& dataSourceCurvePath() const;
+	CLASS_D_ACCESSOR_DECL(QString, dataSourceCurvePath, DataSourceCurvePath)
 
 	POINTER_D_ACCESSOR_DECL(const AbstractColumn, xDataColumn, XDataColumn)
 	POINTER_D_ACCESSOR_DECL(const AbstractColumn, yDataColumn, YDataColumn)
@@ -103,6 +111,9 @@ private Q_SLOTS:
 	void yDataColumnNameChanged();
 	void y2DataColumnNameChanged();
 
+	void dataSourceCurveAboutToBeRemoved(const AbstractAspect*);
+	void dataSourceCurveNameChanged();
+
 Q_SIGNALS:
 	void sourceDataChanged(); // emitted when the source data used in the analysis curves was changed to enable the recalculation in the dock widgets
 	void dataSourceTypeChanged(XYAnalysisCurve::DataSourceType);
@@ -110,6 +121,10 @@ Q_SIGNALS:
 	void xDataColumnChanged(const AbstractColumn*);
 	void yDataColumnChanged(const AbstractColumn*);
 	void y2DataColumnChanged(const AbstractColumn*);
+
+	friend class CommonAnalysisTest;
+	friend class FourierTest;
+	friend class FitTest;
 };
 
 #endif
