@@ -18,6 +18,10 @@ class ParserTest : public CommonTest {
 
 private Q_SLOTS:
 	void testBasics();
+	void testFunction0Arguments();
+	void testFunction1Argument();
+	void testFunction2Arguments();
+	void testFunction3Arguments();
 	void testErrors();
 	void testVariables();
 	void testLocale();
