@@ -18,6 +18,8 @@ class XYCurveTest : public CommonTest {
 
 private Q_SLOTS:
 
+	void setColumn();
+
 	void addUniqueLineTest01();
 
 	// lineSkipGaps = false
@@ -62,6 +64,14 @@ private Q_SLOTS:
 
 	// Hover XYCurve
 	void hooverCurveIntegerEndingZeros();
+
+	void scatterStrictMonotonicIncreasing();
+	void scatterMonotonicIncreasing();
+	void scatterMonotonicIncreasingPlotRangeDecreasing();
+	void scatterMonotonicDecreasing();
+
+	void lineMonotonicIncreasing();
+	void lineMonotonicIncreasingPlotRangeDecreasing();
 };
 
 #endif // XYCURVETEST_H
