@@ -2,6 +2,8 @@
 #include "backend/gsl/ExpressionParser.h"
 #include "backend/gsl/functions.h"
 
+using namespace Parsing;
+
 namespace {
 func_t1 getFunction1(const QString& s) {
 	const QString functionName(s);
@@ -44,6 +46,20 @@ void ExpressionParserTest::testIsValid() {
 	QCOMPARE(ExpressionParser::isValid(expr, vars), true); // should not crash
 }
 
+void ExpressionParserTest::testIsInValid() {
+	const QString expr = QStringLiteral("cell");
+	const QStringList vars = {};
+
+	QCOMPARE(ExpressionParser::isValid(expr, vars), false); // should not crash and should be invalid
+}
+
+void ExpressionParserTest::testIsInValid2() {
+	const QString expr = QStringLiteral("cell_isdk"); // Not a valid symbol
+	const QStringList vars = {};
+
+	QCOMPARE(ExpressionParser::isValid(expr, vars), false); // should not crash and should be invalid
+}
+
 void ExpressionParserTest::testIsValidStdev() {
 	const QString expr = QStringLiteral("stdev(x)");
 	const QStringList vars = {QStringLiteral("x")};
@@ -68,7 +84,7 @@ void ExpressionParserTest::testFunctionArguments2() {
 			QVERIFY(_functions[i].parameterFunction != nullptr);
 	}
 
-	QCOMPARE(ExpressionParser::parameters(functionName), QStringLiteral("(condition; trueValue; falseValue)"));
+	QCOMPARE(ExpressionParser::parameters(functionName), i18n("(%1; %2; %3)", i18n("condition"), i18n("trueValue"), i18n("falseValue")));
 }
 
 void ExpressionParserTest::testUniques() {
@@ -320,7 +336,8 @@ void ExpressionParserTest::testevaluateCartesian() {
 	xVectors << new QVector<double>({1., 2., 3.}); // x
 	xVectors << new QVector<double>({4., 5., 6., 9.}); // y
 	QVector<double> yVector({101., 123., 345., 239., 1290., 43290., 238., 342., 823., 239.});
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QVector<double> ref({5., 7., 9.});
 	QCOMPARE(yVector.size(), 10);
@@ -343,7 +360,8 @@ void ExpressionParserTest::testevaluateCartesianConstExpr() {
 	xVectors << new QVector<double>({1., 2., 3.}); // x
 	xVectors << new QVector<double>({4., 5., 6., 9.}); // y
 	QVector<double> yVector({101., 123., 345., 239., 1290., 43290., 238., 342., 823., 239.});
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 10);
 	// All yVector rows are filled
@@ -365,7 +383,8 @@ void ExpressionParserTest::testEvaluateAnd() {
 		5.,
 		5.,
 	}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 4);
 	QCOMPARE(yVector.at(0), 0.);
@@ -388,7 +407,8 @@ void ExpressionParserTest::testEvaluateOr() {
 		5.,
 		5.,
 	}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 4);
 	QCOMPARE(yVector.at(0), 1.);
@@ -405,7 +425,8 @@ void ExpressionParserTest::testEvaluateNot() {
 
 	xVectors << new QVector<double>({1., 0., -1., 2.}); // x
 	QVector<double> yVector({5., 5., 5., 5.}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 4);
 	QCOMPARE(yVector.at(0), 0.);
@@ -428,7 +449,8 @@ void ExpressionParserTest::testEvaluateLogicalExpression() {
 		5.,
 		5.,
 	}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	// if or evaluated first:
 	// x y
@@ -461,7 +483,8 @@ void ExpressionParserTest::testevaluateGreaterThan() {
 	xVectors << new QVector<double>({0., 1., 0., 1., -1., -std::numeric_limits<double>::infinity(), 1, std::numeric_limits<double>::infinity(), 7.}); // x
 	xVectors << new QVector<double>({0., 0., 1., 1., -2., std::numeric_limits<double>::infinity(), std::nan("0"), 1e9, 7.}); // y
 	QVector<double> yVector({5., 5., 5., 5., 5., 5., 5., 5., 5.}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 9);
 	QCOMPARE(yVector.at(0), 0.);
@@ -484,7 +507,8 @@ void ExpressionParserTest::testevaluateLessThan() {
 	xVectors << new QVector<double>({0., 1., 0., 1., -1., -std::numeric_limits<double>::infinity(), 1, std::numeric_limits<double>::infinity(), 7.}); // x
 	xVectors << new QVector<double>({0., 0., 1., 1., -2., std::numeric_limits<double>::infinity(), std::nan("0"), 1e9, 7.}); // y
 	QVector<double> yVector({5., 5., 5., 5., 5., 5., 5., 5., 5.}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 9);
 	QCOMPARE(yVector.at(0), 0.);
@@ -507,7 +531,8 @@ void ExpressionParserTest::testevaluateLessEqualThan() {
 	xVectors << new QVector<double>({0., 1., 0., 1., -1., -std::numeric_limits<double>::infinity(), 1, std::numeric_limits<double>::infinity(), 7.}); // x
 	xVectors << new QVector<double>({0., 0., 1., 1., -2., std::numeric_limits<double>::infinity(), std::nan("0"), 1e9, 7.}); // y
 	QVector<double> yVector({5., 5., 5., 5., 5., 5., 5., 5., 5.}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 9);
 	QCOMPARE(yVector.at(0), 1.);
@@ -530,7 +555,8 @@ void ExpressionParserTest::testevaluateGreaterEqualThan() {
 	xVectors << new QVector<double>({0., 1., 0., 1., -1., -std::numeric_limits<double>::infinity(), 1, std::numeric_limits<double>::infinity(), 7.}); // x
 	xVectors << new QVector<double>({0., 0., 1., 1., -2., std::numeric_limits<double>::infinity(), std::nan("0"), 1e9, 7.}); // y
 	QVector<double> yVector({5., 5., 5., 5., 5., 5., 5., 5., 5.}); // random value
-	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+	auto* parser = ExpressionParser::getInstance();
+	parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 	QCOMPARE(yVector.size(), 9);
 	QCOMPARE(yVector.at(0), 1.);
@@ -544,6 +570,81 @@ void ExpressionParserTest::testevaluateGreaterEqualThan() {
 	QCOMPARE(yVector.at(8), 1.);
 }
 
+void ExpressionParserTest::testBenchmark() {
+	const QString expr = QStringLiteral("atan2(x;y) + sqrt(x)");
+	const QStringList vars = {QStringLiteral("x"), QStringLiteral("y")};
+
+	const int values = 3000;
+
+	{
+		QVector<QVector<double>*> xVectors;
+
+		auto* x = new QVector<double>(values);
+		auto* y = new QVector<double>(values);
+		for (int i = 0; i < values; i++) {
+			if (i % 2 == 0) {
+				x->operator[](i) = 5.;
+				y->operator[](i) = 2.;
+			} else {
+				x->operator[](i) = 24.;
+				y->operator[](i) = 22.;
+			}
+		}
+
+		xVectors << x; // x
+		xVectors << y; // y
+		QVector<double> yVector(values); // random value
+		QBENCHMARK {
+			auto* parser = ExpressionParser::getInstance();
+			parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector, true);
+		}
+		// QCOMPARE(QLatin1String(Parsing::lastErrorMessage()), QStringLiteral(""));
+
+		QCOMPARE(yVector.size(), values);
+		for (int i = 0; i < values; i++) {
+			if (i % 2 == 0) {
+				VALUES_EQUAL(yVector.at(i), 3.42635792718232);
+			} else {
+				VALUES_EQUAL(yVector.at(i), 5.72782854435533);
+			}
+		}
+	}
+
+	{
+		QVector<QVector<double>*> xVectors;
+
+		auto* x = new QVector<double>(values);
+		auto* y = new QVector<double>(values);
+		for (int i = 0; i < values; i++) {
+			if (i % 2 == 0) {
+				x->operator[](i) = 5.;
+				y->operator[](i) = 2.;
+			} else {
+				x->operator[](i) = 24.;
+				y->operator[](i) = 22.;
+			}
+		}
+
+		xVectors << x; // x
+		xVectors << y; // y
+		QVector<double> yVector(values); // random value
+		QBENCHMARK {
+			auto* parser = ExpressionParser::getInstance();
+			parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector, false);
+		}
+		// QCOMPARE(QLatin1String(Parsing::lastErrorMessage()), QStringLiteral(""));
+
+		QCOMPARE(yVector.size(), values);
+		for (int i = 0; i < values; i++) {
+			if (i % 2 == 0) {
+				VALUES_EQUAL(yVector.at(i), 3.42635792718232);
+			} else {
+				VALUES_EQUAL(yVector.at(i), 5.72782854435533);
+			}
+		}
+	}
+}
+
 // This is not implemented. It uses always the smallest rowCount
 // Does not matter if the variable is used in the expression or not
 // void ExpressionParserTest::testevaluateCartesianConstExpr2() {
@@ -555,7 +656,8 @@ void ExpressionParserTest::testevaluateGreaterEqualThan() {
 //	xVectors << new QVector<double>({1., 2., 3.}); // x
 //	xVectors << new QVector<double>({4., 5., 6., 9.}); // y
 //	QVector<double> yVector({101., 123., 345., 239., 1290., 43290., 238., 342., 823., 239.});
-//	ExpressionParser::evaluateCartesian(expr, vars, xVectors, &yVector);
+//	auto* parser = ExpressionParser::getInstance();
+// parser->tryEvaluateCartesian(expr, vars, xVectors, &yVector);
 
 //	QVector<double> ref({9, 10, 11, 14}); // 5 + y
 //	QCOMPARE(yVector.size(), 10);
