@@ -3,7 +3,7 @@
 	Project              : LabPlot
 	Description          : Tests for statistical plots like Q-Q plot, KDE plot, etc.
 	--------------------------------------------------------------------
-	SPDX-FileCopyrightText: 2023 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2023-2024 Alexander Semke <alexander.semke@web.de>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -38,6 +38,32 @@ private Q_SLOTS:
 	void testBarPlotInit();
 	void testBarPlotDuplicate();
 	void testBarPlotRange();
+
+	// process behavior chart
+	void testPBChartInit();
+	void testPBChartDuplicate();
+
+	void testPBChartXmRAverage();
+	void testPBChartmRAverage();
+	void testPBChartXmRMedian();
+	void testPBChartmRMedian();
+
+	void testPBChartXBarRAverage();
+	void testPBChartRAverage();
+
+	void testPBChartXBarS();
+	void testPBChartS();
+
+	void testPBChartNP();
+	void testPBChartP();
+	void testPBChartC();
+	void testPBChartU();
+
+	// run chart
+	void testRunChartInit();
+	void testRunChartDuplicate();
+	void testRunChartCenterAverage();
+	void testRunChartCenterMedian();
 };
 
 #endif
