@@ -10,6 +10,6 @@ client = socket(AF_INET, SOCK_DGRAM)
 client.bind(ADDR)
 while True:
   data, addr = client.recvfrom(1024)
-  print 'read ' + data, addr
+  print('read ' + data, addr)
 
 
