@@ -6,7 +6,7 @@ ADDR = './local_socket'
 serv = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
 
 if os.path.exists(ADDR):
-  print 'socket exists. Removing it'
+  print('socket exists. Removing it')
   os.remove(ADDR)
 
 serv.bind(ADDR)
@@ -19,6 +19,6 @@ while True:
   print 'client connected ... ', addr
   cpu_percent = str(psutil.cpu_percent())
   conn.send(cpu_percent)
-  print 'written ' + cpu_percent
+  print('written ' + cpu_percent)
   conn.close()
-  print 'client disconnected'
+  print('client disconnected')
