@@ -10,9 +10,9 @@
 #ifndef READSTATFILTERTEST_H
 #define READSTATFILTERTEST_H
 
-#include "../../CommonTest.h"
+#include "../../CommonMetaTest.h"
 
-class ReadStatFilterTest : public CommonTest {
+class ReadStatFilterTest : public CommonMetaTest {
 	Q_OBJECT
 
 private Q_SLOTS:
