@@ -10,13 +10,13 @@
 #ifndef IMPORTSQLDATABASETEST_H
 #define IMPORTSQLDATABASETEST_H
 
-#include "../../CommonTest.h"
+#include "../../CommonMetaTest.h"
 
-class ImportSqlDatabaseTest : public CommonTest {
+class ImportSqlDatabaseTest : public CommonMetaTest {
 	Q_OBJECT
 
 private Q_SLOTS:
-	void initTestCase();
+	void initTestCase() override;
 
 	// import full table
 	void testFullTableReplace();
