@@ -10,9 +10,9 @@
 #ifndef XLSXFILTERTEST_H
 #define XLSXFILTERTEST_H
 
-#include "../../CommonTest.h"
+#include "../../CommonMetaTest.h"
 
-class XLSXFilterTest : public CommonTest {
+class XLSXFilterTest : public CommonMetaTest {
 	Q_OBJECT
 
 private Q_SLOTS:
@@ -22,5 +22,6 @@ private Q_SLOTS:
 	void importFile3ColsStartEndColumn();
 	void importFileEmptyCells();
 	void importFileDatetime();
+	void importFormula();
 };
 #endif
