@@ -1,7 +1,7 @@
 /*
 	File                 : MultiRangeTest.cpp
 	Project              : LabPlot
-	Description          : Tests for multi ranges
+	Description          : Tests for multi ranges, part 1
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2021 Martin Marmsoler <martin.marmsoler@gmail.com>
 	SPDX-FileCopyrightText: 2021 Stefan Gerlach <stefan.gerlach@uni.kn>
@@ -10,105 +10,11 @@
 */
 
 #include "MultiRangeTest.h"
+#include "MultiRangeTest_macros.h"
 
-#include "backend/core/Project.h"
-#include "backend/core/Workbook.h"
-#include "backend/lib/macros.h"
-#include "backend/matrix/Matrix.h"
-#include "backend/spreadsheet/Spreadsheet.h"
-#include "backend/worksheet/Worksheet.h"
-#include "backend/worksheet/plots/cartesian/Axis.h"
-#include "backend/worksheet/plots/cartesian/CartesianCoordinateSystem.h"
-#include "backend/worksheet/plots/cartesian/CartesianPlot.h"
-#include "backend/worksheet/plots/cartesian/CartesianPlotPrivate.h"
-#include "backend/worksheet/plots/cartesian/XYCurve.h"
-#include "commonfrontend/worksheet/WorksheetView.h"
-#include "kdefrontend/dockwidgets/BaseDock.h"
-#include "kdefrontend/dockwidgets/XYCurveDock.h"
-
-#include <QAction>
-#include <QComboBox>
-#include <QGraphicsSceneWheelEvent>
-
-//##############################################################################
-//#####################  import of LabPlot projects ############################
-//##############################################################################
-
-#define LOAD_PROJECT                                                                                                                                           \
-	Project project;                                                                                                                                           \
-	project.load(QFINDTESTDATA(QLatin1String("data/TestMultiRange.lml")));                                                                                     \
-	/* check the project tree for the imported project */                                                                                                      \
-	/* first child of the root folder */                                                                                                                       \
-	auto* aspect = project.child<AbstractAspect>(0);                                                                                                           \
-	QVERIFY(aspect != nullptr);                                                                                                                                \
-	if (aspect)                                                                                                                                                \
-		QCOMPARE(aspect->name(), QLatin1String("Arbeitsblatt"));                                                                                               \
-	QVERIFY(aspect->type() == AspectType::Worksheet);                                                                                                          \
-	auto w = dynamic_cast<Worksheet*>(aspect);                                                                                                                 \
-	if (!w)                                                                                                                                                    \
-		return;                                                                                                                                                \
-                                                                                                                                                               \
-	auto p1 = dynamic_cast<CartesianPlot*>(aspect->child<CartesianPlot>(0));                                                                                   \
-	QVERIFY(p1 != nullptr);                                                                                                                                    \
-	auto p2 = dynamic_cast<CartesianPlot*>(aspect->child<CartesianPlot>(1));                                                                                   \
-	QVERIFY(p2 != nullptr);                                                                                                                                    \
-	if (!p1 || !p2)                                                                                                                                            \
-		return;                                                                                                                                                \
-                                                                                                                                                               \
-	auto* view = dynamic_cast<WorksheetView*>(w->view());                                                                                                      \
-	QVERIFY(view != nullptr);                                                                                                                                  \
-	view->initActions(); /* needed by SET_CARTESIAN_MOUSE_MODE() */                                                                                            \
-                                                                                                                                                               \
-	/* axis selected */                                                                                                                                        \
-	auto sinCurve = dynamic_cast<XYCurve*>(p1->child<XYCurve>(0));                                                                                             \
-	QVERIFY(sinCurve != nullptr);                                                                                                                              \
-	if (!sinCurve)                                                                                                                                             \
-		return;                                                                                                                                                \
-	QCOMPARE(sinCurve->name(), QStringLiteral("sinCurve"));                                                                                                    \
-	auto tanCurve = dynamic_cast<XYCurve*>(p1->child<XYCurve>(1));                                                                                             \
-	QVERIFY(tanCurve != nullptr);                                                                                                                              \
-	if (!tanCurve)                                                                                                                                             \
-		return;                                                                                                                                                \
-	QCOMPARE(tanCurve->name(), QStringLiteral("tanCurve"));                                                                                                    \
-	auto logCurve = dynamic_cast<XYCurve*>(p1->child<XYCurve>(2));                                                                                             \
-	QVERIFY(logCurve != nullptr);                                                                                                                              \
-	if (!logCurve)                                                                                                                                             \
-		return;                                                                                                                                                \
-	QCOMPARE(logCurve->name(), QStringLiteral("logx"));                                                                                                        \
-                                                                                                                                                               \
-	auto cosCurve = dynamic_cast<XYCurve*>(p2->child<XYCurve>(0));                                                                                             \
-	QVERIFY(cosCurve != nullptr);                                                                                                                              \
-	if (!cosCurve)                                                                                                                                             \
-		return;                                                                                                                                                \
-	QCOMPARE(cosCurve->name(), QStringLiteral("cosCurve"));                                                                                                    \
-                                                                                                                                                               \
-	auto horAxisP1 = static_cast<Axis*>(p1->child<Axis>(0));                                                                                                   \
-	QVERIFY(horAxisP1 != nullptr);                                                                                                                             \
-	QCOMPARE(horAxisP1->orientation() == Axis::Orientation::Horizontal, true);                                                                                 \
-                                                                                                                                                               \
-	auto vertAxisP1 = static_cast<Axis*>(p1->child<Axis>(1));                                                                                                  \
-	QVERIFY(vertAxisP1 != nullptr);                                                                                                                            \
-	QCOMPARE(vertAxisP1->orientation() == Axis::Orientation::Vertical, true);                                                                                  \
-                                                                                                                                                               \
-	auto vertAxis2P1 = static_cast<Axis*>(p1->child<Axis>(2));                                                                                                 \
-	QVERIFY(vertAxis2P1 != nullptr);                                                                                                                           \
-	QCOMPARE(vertAxis2P1->orientation() == Axis::Orientation::Vertical, true);                                                                                 \
-                                                                                                                                                               \
-	auto vertAxis3P1 = static_cast<Axis*>(p1->child<Axis>(3));                                                                                                 \
-	QVERIFY(vertAxis3P1 != nullptr);                                                                                                                           \
-	QCOMPARE(vertAxis3P1->orientation() == Axis::Orientation::Vertical, true);                                                                                 \
-	QCOMPARE(vertAxis3P1->name(), QStringLiteral("y-axis 1"));                                                                                                 \
-                                                                                                                                                               \
-	auto horAxisP2 = static_cast<Axis*>(p2->child<Axis>(0));                                                                                                   \
-	QVERIFY(horAxisP2 != nullptr);                                                                                                                             \
-	QCOMPARE(horAxisP2->orientation() == Axis::Orientation::Horizontal, true);
-
-#define SET_CARTESIAN_MOUSE_MODE(mode)                                                                                                                         \
-	QAction a(nullptr);                                                                                                                                        \
-	a.setData(static_cast<int>(mode));                                                                                                                         \
-	view->cartesianPlotMouseModeChanged(&a);
-
-////////////////////////////////////////////////////////////////
+// ##############################################################################
+// #####################  import of LabPlot projects ############################
+// ##############################################################################
 
 // Test1:
 // Check if the correct actions are enabled/disabled.
@@ -389,6 +295,131 @@ void MultiRangeTest::zoomInX_SingleRange() {
 	COMPARE_DOUBLE_VECTORS(vertAxis3P1->tickLabelValues(), refValuesAxis3); // on third axis there is no autoscale, because it uses a different range
 }
 
+void MultiRangeTest::zoomInX_SingleRangeDateTimeMonotonicIncrease() {
+	Project project;
+	auto* ws = new Worksheet(QStringLiteral("worksheet"));
+	QVERIFY(ws != nullptr);
+	project.addChild(ws);
+
+	auto* view = dynamic_cast<WorksheetView*>(ws->view());
+	QVERIFY(view != nullptr);
+	view->initActions(); // needed by SET_CARTESIAN_MOUSE_MODE()
+
+	Spreadsheet* spreadsheetData = new Spreadsheet(QStringLiteral("data"), false);
+	spreadsheetData->setColumnCount(2);
+	spreadsheetData->setRowCount(3);
+	project.addChild(spreadsheetData);
+	auto* xCol = spreadsheetData->column(0);
+	xCol->setColumnMode(AbstractColumn::ColumnMode::DateTime);
+	QDateTime dt1 = QDateTime::fromString(QStringLiteral("2017-07-10T00:00:00Z"), Qt::ISODate);
+	QDateTime dt2 = QDateTime::fromString(QStringLiteral("2017-07-11T00:00:00Z"), Qt::ISODate);
+	QDateTime dt3 = QDateTime::fromString(QStringLiteral("2017-07-12T00:00:00Z"), Qt::ISODate);
+	QDateTime dt4 = QDateTime::fromString(QStringLiteral("2017-07-13T00:00:00Z"), Qt::ISODate);
+	QDateTime dt5 = QDateTime::fromString(QStringLiteral("2017-07-14T00:00:00Z"), Qt::ISODate);
+	QDateTime dt6 = QDateTime::fromString(QStringLiteral("2017-07-15T00:00:00Z"), Qt::ISODate);
+	QDateTime dt7 = QDateTime::fromString(QStringLiteral("2017-07-16T00:00:00Z"), Qt::ISODate);
+	QDateTime dt8 = QDateTime::fromString(QStringLiteral("2017-07-17T00:00:00Z"), Qt::ISODate);
+	QDateTime dt9 = QDateTime::fromString(QStringLiteral("2017-07-18T00:00:00Z"), Qt::ISODate);
+	QDateTime dt10 = QDateTime::fromString(QStringLiteral("2017-07-19T00:00:00Z"), Qt::ISODate);
+	xCol->replaceDateTimes(-1, QVector<QDateTime>({dt1, dt2, dt3, dt4, dt5, dt6, dt7, dt8, dt9, dt10}));
+	auto* yCol = spreadsheetData->column(1);
+	yCol->replaceValues(-1, QVector<double>({2., 3., 4., 5., 6., 7., 8., 9., 10., 11.}));
+
+	auto* p = new CartesianPlot(QStringLiteral("plot"));
+	p->setType(CartesianPlot::Type::TwoAxes); // Otherwise no axis are created
+	p->setNiceExtend(false);
+	QVERIFY(p != nullptr);
+	ws->addChild(p);
+
+	auto* curve = new XYCurve(QStringLiteral("curve"));
+	p->addChild(curve);
+	curve->setXColumn(xCol);
+	curve->setYColumn(yCol);
+
+	CHECK_RANGE(p, curve, Dimension::X, (double)dt1.toMSecsSinceEpoch(), (double)dt10.toMSecsSinceEpoch());
+	CHECK_RANGE(p, curve, Dimension::Y, 2., 11.);
+
+	QCOMPARE(p->rangeCount(Dimension::X), 1);
+	QCOMPARE(p->rangeCount(Dimension::Y), 1);
+	QCOMPARE(p->range(Dimension::Y, 0).autoScale(), true);
+
+	const auto& axes = p->children<Axis>();
+	QCOMPARE(axes.count(), 2);
+	QCOMPARE(axes.at(0)->name(), QStringLiteral("x"));
+	axes.at(0)->setSelected(true);
+	SET_CARTESIAN_MOUSE_MODE(CartesianPlot::MouseMode::ZoomXSelection)
+
+	p->mousePressZoomSelectionMode(QPointF((double)dt3.toMSecsSinceEpoch(), 3.), 0);
+	p->mouseMoveZoomSelectionMode(QPointF((double)dt5.toMSecsSinceEpoch(), 3.), 0);
+	p->mouseReleaseZoomSelectionMode(0);
+
+	CHECK_RANGE(p, curve, Dimension::X, (double)dt3.toMSecsSinceEpoch(), (double)dt5.toMSecsSinceEpoch()); // zoom
+	CHECK_RANGE(p, curve, Dimension::Y, 4., 6.); // autoscaled
+}
+
+void MultiRangeTest::zoomInX_SingleRangeDateTimeNonMonotonic() {
+	Project project;
+	auto* ws = new Worksheet(QStringLiteral("worksheet"));
+	QVERIFY(ws != nullptr);
+	project.addChild(ws);
+
+	auto* view = dynamic_cast<WorksheetView*>(ws->view());
+	QVERIFY(view != nullptr);
+	view->initActions(); // needed by SET_CARTESIAN_MOUSE_MODE()
+
+	Spreadsheet* spreadsheetData = new Spreadsheet(QStringLiteral("data"), false);
+	spreadsheetData->setColumnCount(2);
+	spreadsheetData->setRowCount(3);
+	project.addChild(spreadsheetData);
+	auto* xCol = spreadsheetData->column(0);
+	xCol->setColumnMode(AbstractColumn::ColumnMode::DateTime);
+	QDateTime dt1 = QDateTime::fromString(QStringLiteral("2017-07-10T00:00:00Z"), Qt::ISODate);
+	QDateTime dt2 = QDateTime::fromString(QStringLiteral("2017-07-11T00:00:00Z"), Qt::ISODate);
+	QDateTime dt3 = QDateTime::fromString(QStringLiteral("2017-07-12T00:00:00Z"), Qt::ISODate);
+	QDateTime dt4 = QDateTime::fromString(QStringLiteral("2017-07-15T00:00:00Z"), Qt::ISODate);
+	QDateTime dt5 = QDateTime::fromString(QStringLiteral("2017-07-14T00:00:00Z"), Qt::ISODate); // Nonmonoton
+	QDateTime dt6 = QDateTime::fromString(QStringLiteral("2017-07-15T00:00:00Z"), Qt::ISODate);
+	QDateTime dt7 = QDateTime::fromString(QStringLiteral("2017-07-16T00:00:00Z"), Qt::ISODate);
+	QDateTime dt8 = QDateTime::fromString(QStringLiteral("2017-07-17T00:00:00Z"), Qt::ISODate);
+	QDateTime dt9 = QDateTime::fromString(QStringLiteral("2017-07-18T00:00:00Z"), Qt::ISODate);
+	QDateTime dt10 = QDateTime::fromString(QStringLiteral("2017-07-19T00:00:00Z"), Qt::ISODate);
+	xCol->replaceDateTimes(-1, QVector<QDateTime>({dt1, dt2, dt3, dt4, dt5, dt6, dt7, dt8, dt9, dt10}));
+	auto* yCol = spreadsheetData->column(1);
+	yCol->replaceValues(-1, QVector<double>({2., 3., 4., 5., 6., 7., 8., 9., 10., 11.}));
+
+	auto* p = new CartesianPlot(QStringLiteral("plot"));
+	p->setType(CartesianPlot::Type::TwoAxes); // Otherwise no axis are created
+	p->setNiceExtend(false);
+	QVERIFY(p != nullptr);
+	ws->addChild(p);
+
+	auto* curve = new XYCurve(QStringLiteral("curve"));
+	p->addChild(curve);
+	curve->setXColumn(xCol);
+	curve->setYColumn(yCol);
+
+	CHECK_RANGE(p, curve, Dimension::X, (double)dt1.toMSecsSinceEpoch(), (double)dt10.toMSecsSinceEpoch());
+	CHECK_RANGE(p, curve, Dimension::Y, 2., 11.);
+
+	QCOMPARE(p->rangeCount(Dimension::X), 1);
+	QCOMPARE(p->rangeCount(Dimension::Y), 1);
+	QCOMPARE(p->range(Dimension::Y, 0).autoScale(), true);
+
+	const auto& axes = p->children<Axis>();
+	QCOMPARE(axes.count(), 2);
+	QCOMPARE(axes.at(0)->name(), QStringLiteral("x"));
+	axes.at(0)->setSelected(true);
+	SET_CARTESIAN_MOUSE_MODE(CartesianPlot::MouseMode::ZoomXSelection)
+
+	p->mousePressZoomSelectionMode(QPointF((double)dt3.toMSecsSinceEpoch(), 3.), 0);
+	p->mouseMoveZoomSelectionMode(QPointF((double)dt5.addSecs(3600 * 2).toMSecsSinceEpoch(), 3.),
+								  0); // Adding an offset, because the error happens only if the exact time is not hit
+	p->mouseReleaseZoomSelectionMode(0);
+
+	CHECK_RANGE(p, curve, Dimension::X, (double)dt3.toMSecsSinceEpoch(), (double)dt5.addSecs(3600 * 2).toMSecsSinceEpoch()); // zoom
+	CHECK_RANGE(p, curve, Dimension::Y, 4., 6.); // autoscaled
+}
+
 void MultiRangeTest::zoomInX_AllRanges() {
 	LOAD_PROJECT
 	auto refValuesAxis1 = vertAxisP1->tickLabelValues();
@@ -857,282 +888,4 @@ void MultiRangeTest::shiftDown_AllRanges() {
 	COMPARE_DOUBLE_VECTORS(vertAxis3P1->tickLabelValues(), ref); // vertAxis3 is not autoscaled when loading, after autoscaling the values are different
 }
 
-void MultiRangeTest::autoScaleYAfterZoomInX() {
-	/* 1) Zoom in X
-	 * 2) Autoscale X
-	 * 3) Check that y also changed! */
-	LOAD_PROJECT
-	auto refValues = horAxisP1->tickLabelValues();
-	horAxisP1->setSelected(true);
-	SET_CARTESIAN_MOUSE_MODE(CartesianPlot::MouseMode::ZoomXSelection)
-
-	p1->mousePressZoomSelectionMode(QPointF(0.2, -150), 0);
-	p1->mouseMoveZoomSelectionMode(QPointF(0.6, 100), 0);
-	p1->mouseReleaseZoomSelectionMode(0);
-
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0.2, 0.6); // zoom
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-
-	p1->navigate(tanCurve->coordinateSystemIndex(), CartesianPlot::NavigationOperation::ScaleAutoX);
-
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, horAxisP1, Dimension::X, 0., 1.); // range is changed in retransform scale
-
-	// retransform of horAxisP1 is done, so the tickLabelValues change back
-	// to be in the range of 0, 1
-	COMPARE_DOUBLE_VECTORS(horAxisP1->tickLabelValues(), refValues);
-}
-
-void MultiRangeTest::autoScaleXAfterZoomInY() {
-	LOAD_PROJECT
-	auto refValues = vertAxisP1->tickLabelValues();
-	vertAxisP1->setSelected(true);
-	SET_CARTESIAN_MOUSE_MODE(CartesianPlot::MouseMode::ZoomYSelection)
-
-	p1->mousePressZoomSelectionMode(QPointF(0.2, -150), 0);
-	p1->mouseMoveZoomSelectionMode(QPointF(0.6, 100), 0);
-	p1->mouseReleaseZoomSelectionMode(0);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -150., 100.); // zoom
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-
-	p1->navigate(tanCurve->coordinateSystemIndex(), CartesianPlot::NavigationOperation::ScaleAutoY);
-
-	// retransform of vertAxisP1 is done, so the tickLabelValues change back
-	COMPARE_DOUBLE_VECTORS(vertAxisP1->tickLabelValues(), refValues);
-}
-
-void MultiRangeTest::baseDockSetAspects_NoPlotRangeChange() {
-	LOAD_PROJECT
-
-	const int sinCurveCSystemIndex = sinCurve->coordinateSystemIndex();
-	const int tanCurveCSystemIndex = tanCurve->coordinateSystemIndex();
-	QVERIFY(sinCurveCSystemIndex != tanCurveCSystemIndex);
-	// checks directly the plot. In the basedock the element is used and not the plot, so do it here too
-	QVERIFY(sinCurve->coordinateSystemCount() == 3);
-
-	XYCurveDock dock(nullptr);
-	dock.setupGeneral();
-	dock.setCurves(QList<XYCurve*>({sinCurve, tanCurve}));
-
-	dock.updatePlotRanges();
-
-	// The coordinatesystem indices shall not change
-	QCOMPARE(sinCurveCSystemIndex, sinCurve->coordinateSystemIndex());
-	QCOMPARE(tanCurveCSystemIndex, tanCurve->coordinateSystemIndex());
-}
-
-/*!
- * \brief MultiRangeTest::mouseWheelXAxisApplyToAllX
- * If applied to all x is activated, using the mousewheel on a
- * selected axis should also execute the mousewheel on other plots
- */
-void MultiRangeTest::mouseWheelXAxisApplyToAllX() {
-	LOAD_PROJECT
-
-	QCOMPARE(w->cartesianPlotActionMode(), Worksheet::CartesianPlotActionMode::ApplyActionToAllX);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.); // zoom
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-
-	horAxisP1->setSelected(true); // seems not to work
-	view->m_selectedElement = horAxisP1;
-
-	int counter = 0;
-	connect(p1, &CartesianPlot::wheelEventSignal, [&counter](int delta, int xIndex, int /*yIndex*/, bool considerDimension, Dimension dim) {
-		QCOMPARE(delta, 10);
-		QCOMPARE(xIndex, 0); // x Range of horAxisP1
-		QCOMPARE(considerDimension, true);
-		QCOMPARE(dim, Dimension::X);
-		counter++;
-	});
-
-	QGraphicsSceneWheelEvent event;
-	event.setDelta(10);
-	p1->d_func()->wheelEvent(&event);
-
-	QCOMPARE(counter, 1);
-
-	// All x ranges are zoomed, for plot 1 and plot 2
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0.1, 0.9); // zoom
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0.1, 0.9); // zoom
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 10., 90.); // zoom
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0.1, 0.9);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0.1, 0.9);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-}
-
-/*!
- * \brief MultiRangeTest::mouseWheelXAxisApplyToAllX
- * If applied to all x is activated, using the mousewheel on a
- * selected axis should also execute the mousewheel on other plots
- * This time the second x axis is used. In the second plot no second x axis is used
- * so check that application does not crash
- */
-void MultiRangeTest::mouseWheelTanCurveApplyToAllX() {
-	LOAD_PROJECT
-
-	QCOMPARE(w->cartesianPlotActionMode(), Worksheet::CartesianPlotActionMode::ApplyActionToAllX);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.); // zoom
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-
-	tanCurve->setSelected(true); // seems not to work
-	view->m_selectedElement = tanCurve;
-
-	int counter = 0;
-	connect(p1, &CartesianPlot::wheelEventSignal, [&counter](int delta, int xIndex, int yIndex, bool considerDimension, Dimension dim) {
-		Q_UNUSED(yIndex);
-		Q_UNUSED(dim);
-		QCOMPARE(delta, 10);
-		QCOMPARE(xIndex, 0); // tan curve has xIndex 0
-		QCOMPARE(considerDimension, false);
-		counter++;
-	});
-
-	QGraphicsSceneWheelEvent event;
-	event.setDelta(10);
-	p1->d_func()->wheelEvent(&event);
-
-	QCOMPARE(counter, 1);
-
-	// All x ranges are zoomed, for plot 1 and plot 2
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0.1, 0.9); // zoom
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0.1, 0.9); // zoom
-	// zoomed in, because with scrolling both axes are scrolled
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -200., 200.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 10., 90.); // zoom
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0.1, 0.9);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0.1, 0.9);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-}
-
-void MultiRangeTest::mouseWheelXAxisApplyToSelected() {
-	LOAD_PROJECT
-
-	w->setCartesianPlotActionMode(Worksheet::CartesianPlotActionMode::ApplyActionToSelection);
-	QCOMPARE(w->cartesianPlotActionMode(), Worksheet::CartesianPlotActionMode::ApplyActionToSelection);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.); // zoom
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-
-	horAxisP1->setSelected(true); // seems not to work
-	view->m_selectedElement = horAxisP1;
-
-	int counter = 0;
-	connect(p1, &CartesianPlot::wheelEventSignal, [&counter](int delta, int xIndex, int /*yIndex*/, bool considerDimension, Dimension dim) {
-		QCOMPARE(delta, 10);
-		QCOMPARE(xIndex, 0); // x Range of horAxisP1
-		QCOMPARE(considerDimension, true);
-		QCOMPARE(dim, Dimension::X);
-		counter++;
-	});
-
-	QGraphicsSceneWheelEvent event;
-	event.setDelta(10);
-	p1->d_func()->wheelEvent(&event);
-
-	QCOMPARE(counter, 1);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0.1, 0.9); // zoom
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0.1, 0.9); // zoom
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.); // Not zoomed
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.); // Not zoomed
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.); // Not zoomed
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-}
-
-void MultiRangeTest::axisMouseMoveApplyToAllX() {
-	LOAD_PROJECT
-
-	w->setCartesianPlotActionMode(Worksheet::CartesianPlotActionMode::ApplyActionToAllX);
-	QCOMPARE(w->cartesianPlotActionMode(), Worksheet::CartesianPlotActionMode::ApplyActionToAllX);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-
-	const int delta = -10; // delta > 0 --> right or up
-	horAxisP1->shiftSignal(delta, Dimension::X, p1->coordinateSystem(horAxisP1->coordinateSystemIndex())->index(Dimension::X));
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0.1, 1.1); // shift
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0.1, 1.1); // shift
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 10., 110.); // shift
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0.1, 1.1); // shift
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0.1, 1.1); // shift
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-}
-
-void MultiRangeTest::axisMouseMoveApplyToSelection() {
-	LOAD_PROJECT
-
-	w->setCartesianPlotActionMode(Worksheet::CartesianPlotActionMode::ApplyActionToSelection);
-	QCOMPARE(w->cartesianPlotActionMode(), Worksheet::CartesianPlotActionMode::ApplyActionToSelection);
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-
-	const int delta = -10; // delta > 0 --> right or up
-	horAxisP1->shiftSignal(delta, Dimension::X, p1->coordinateSystem(horAxisP1->coordinateSystemIndex())->index(Dimension::X));
-
-	CHECK_RANGE(p1, sinCurve, Dimension::X, 0.1, 1.1); // shift
-	CHECK_RANGE(p1, sinCurve, Dimension::Y, -1., 1.);
-	CHECK_RANGE(p1, tanCurve, Dimension::X, 0.1, 1.1); // shift
-	CHECK_RANGE(p1, tanCurve, Dimension::Y, -250., 250.);
-	CHECK_RANGE(p1, logCurve, Dimension::X, 0., 100.);
-	CHECK_RANGE(p1, logCurve, Dimension::Y, -10., 6.);
-	CHECK_RANGE(p2, horAxisP1, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::X, 0., 1.);
-	CHECK_RANGE(p2, cosCurve, Dimension::Y, -1., 1.);
-}
-
 QTEST_MAIN(MultiRangeTest)
